@@ -157,7 +157,7 @@ Sanitizer.prototype = {
 
         // Clear plugin data.
         let ph = Cc["@mozilla.org/plugin/host;1"].getService(Ci.nsIPluginHost);
-        const phInterface = Ci.nsIPluginHost_MOZILLA_2_0_BRANCH;
+        const phInterface = Ci.nsIPluginHost;
         const FLAG_CLEAR_ALL = phInterface.FLAG_CLEAR_ALL;
         ph.QueryInterface(phInterface);
 
