@@ -63,12 +63,15 @@
 
 GK_ATOM(_empty, "")
 GK_ATOM(moz, "_moz")
+GK_ATOM(moztype, "_moz-type")
 GK_ATOM(mozdirty, "_moz_dirty")
+GK_ATOM(mozdonotsend, "moz-do-not-send")
 GK_ATOM(mozeditorbogusnode, "_moz_editor_bogus_node")
 GK_ATOM(mozgeneratedcontentbefore, "_moz_generated_content_before")
 GK_ATOM(mozgeneratedcontentafter, "_moz_generated_content_after")
 GK_ATOM(mozgeneratedcontentimage, "_moz_generated_content_image")
 GK_ATOM(_moz_target, "_moz_target")
+GK_ATOM(_moz_type, "_moz-type")
 GK_ATOM(menuactive, "_moz-menuactive")
 GK_ATOM(_poundDefault, "#default")
 GK_ATOM(_asterix, "*")
@@ -108,7 +111,9 @@ GK_ATOM(applyImports, "apply-imports")
 GK_ATOM(applyTemplates, "apply-templates")
 GK_ATOM(archive, "archive")
 GK_ATOM(area, "area")
+GK_ATOM(article, "article")
 GK_ATOM(ascending, "ascending")
+GK_ATOM(aside, "aside")
 GK_ATOM(aspectRatio, "aspect-ratio")
 GK_ATOM(assign, "assign")
 GK_ATOM(async, "async")
@@ -116,11 +121,9 @@ GK_ATOM(attribute, "attribute")
 GK_ATOM(attributeSet, "attribute-set")
 GK_ATOM(aural, "aural")
 GK_ATOM(_auto, "auto")
-#ifdef MOZ_MEDIA
-GK_ATOM(autobuffer, "autobuffer")
-#endif
 GK_ATOM(autocheck, "autocheck")
 GK_ATOM(autocomplete, "autocomplete")
+GK_ATOM(autofocus, "autofocus")
 #ifdef MOZ_MEDIA
 GK_ATOM(autoplay, "autoplay")
 #endif
@@ -130,7 +133,6 @@ GK_ATOM(b, "b")
 GK_ATOM(background, "background")
 GK_ATOM(base, "base")
 GK_ATOM(basefont, "basefont")
-GK_ATOM(baseLevel, "BaseLevel")                            // PRUint8
 GK_ATOM(baseline, "baseline")
 GK_ATOM(bdo, "bdo")
 GK_ATOM(before, "before")
@@ -185,12 +187,12 @@ GK_ATOM(_char, "char")
 GK_ATOM(charcode, "charcode")
 GK_ATOM(charoff, "charoff")
 GK_ATOM(charset, "charset")
-GK_ATOM(charType, "charType")                              // PRUint8
 GK_ATOM(checkbox, "checkbox")
 GK_ATOM(checked, "checked")
 GK_ATOM(child, "child")
 GK_ATOM(children, "children")
 GK_ATOM(choose, "choose")
+GK_ATOM(chromemargin, "chromemargin")
 GK_ATOM(circ, "circ")
 GK_ATOM(circle, "circle")
 GK_ATOM(cite, "cite")
@@ -199,9 +201,9 @@ GK_ATOM(classid, "classid")
 GK_ATOM(clear, "clear")
 GK_ATOM(click, "click")
 GK_ATOM(clickcount, "clickcount")
+GK_ATOM(clickthrough, "clickthrough")
 GK_ATOM(movetoclick, "movetoclick")
 GK_ATOM(clip, "clip")
-GK_ATOM(clonedTextForPrint, "clonedTextForPrint")
 GK_ATOM(close, "close")
 GK_ATOM(closed, "closed")
 GK_ATOM(closemenu, "closemenu")
@@ -242,7 +244,6 @@ GK_ATOM(contentLocation, "content-location")
 GK_ATOM(headerContentScriptType, "content-script-type")
 GK_ATOM(headerContentStyleType, "content-style-type")
 GK_ATOM(headerContentType, "content-type")
-GK_ATOM(headerXFO, "x-frame-options")
 GK_ATOM(context, "context")
 GK_ATOM(contextmenu, "contextmenu")
 GK_ATOM(contextmenulistener, "contextmenulistener")
@@ -257,11 +258,10 @@ GK_ATOM(count, "count")
 GK_ATOM(crop, "crop")
 GK_ATOM(curpos, "curpos")
 GK_ATOM(current, "current")
-#ifdef MOZ_MEDIA
 GK_ATOM(currentloop, "currentloop")
-#endif
 GK_ATOM(cycler, "cycler")
 GK_ATOM(data, "data")
+GK_ATOM(datalist, "datalist")
 GK_ATOM(dataType, "data-type")
 GK_ATOM(datasources, "datasources")
 GK_ATOM(datetime, "datetime")
@@ -290,6 +290,7 @@ GK_ATOM(descendantOrSelf, "descendant-or-self")
 GK_ATOM(descending, "descending")
 GK_ATOM(description, "description")
 GK_ATOM(destructor, "destructor")
+GK_ATOM(details, "details")
 GK_ATOM(deviceAspectRatio, "device-aspect-ratio")
 GK_ATOM(deviceHeight, "device-height")
 GK_ATOM(deviceWidth, "device-width")
@@ -328,6 +329,7 @@ GK_ATOM(dragleave, "dragleave")
 GK_ATOM(dragover, "dragover")
 GK_ATOM(dragSession, "dragSession")
 GK_ATOM(dragstart, "dragstart")
+GK_ATOM(drawintitlebar, "drawintitlebar")
 GK_ATOM(drop, "drop")
 GK_ATOM(dropAfter, "dropAfter")
 GK_ATOM(dropBefore, "dropBefore")
@@ -342,7 +344,6 @@ GK_ATOM(elementAvailable, "element-available")
 GK_ATOM(elements, "elements")
 GK_ATOM(em, "em")
 GK_ATOM(embed, "embed")
-GK_ATOM(embeddingLevel, "EmbeddingLevel")                  // PRUint8
 GK_ATOM(embossed, "embossed")
 GK_ATOM(empty, "empty")
 GK_ATOM(encoding, "encoding")
@@ -350,7 +351,6 @@ GK_ATOM(enctype, "enctype")
 GK_ATOM(end, "end")
 GK_ATOM(end_after, "end_after")
 GK_ATOM(end_before, "end_before")
-GK_ATOM(endsInDiacritic, "EndsInDiacritic")                // PRUint32
 GK_ATOM(equalsize, "equalsize")
 GK_ATOM(error, "error")
 GK_ATOM(even, "even")
@@ -368,11 +368,15 @@ GK_ATOM(_false, "false")
 GK_ATOM(farthest, "farthest")
 GK_ATOM(field, "field")
 GK_ATOM(fieldset, "fieldset")
+GK_ATOM(figcaption, "figcaption")
+GK_ATOM(figure, "figure")
 GK_ATOM(fixed, "fixed")
 GK_ATOM(fixedList, "Fixed-list")
 GK_ATOM(flags, "flags")
 GK_ATOM(flex, "flex")
 GK_ATOM(flexgroup, "flexgroup")
+GK_ATOM(flip, "flip")
+GK_ATOM(floating, "floating")
 GK_ATOM(floatList, "Float-list")
 GK_ATOM(floor, "floor")
 GK_ATOM(focus, "focus")
@@ -381,11 +385,17 @@ GK_ATOM(followingSibling, "following-sibling")
 GK_ATOM(font, "font")
 GK_ATOM(fontWeight, "font-weight")
 GK_ATOM(fontpicker, "fontpicker")
+GK_ATOM(footer, "footer")
 GK_ATOM(_for, "for")
 GK_ATOM(forEach, "for-each")
 GK_ATOM(form, "form")
+GK_ATOM(formaction, "formaction")
 GK_ATOM(format, "format")
 GK_ATOM(formatNumber, "format-number")
+GK_ATOM(formenctype, "formenctype")
+GK_ATOM(formmethod, "formmethod")
+GK_ATOM(formnovalidate, "formnovalidate")
+GK_ATOM(formtarget, "formtarget")
 GK_ATOM(frame, "frame")
 GK_ATOM(frameborder, "frameborder")
 GK_ATOM(frameset, "frameset")
@@ -414,10 +424,13 @@ GK_ATOM(HARD, "HARD")
 GK_ATOM(hasSameNode, "has-same-node")
 GK_ATOM(hbox, "hbox")
 GK_ATOM(head, "head")
+GK_ATOM(header, "header")
 GK_ATOM(headers, "headers")
 GK_ATOM(height, "height")
+GK_ATOM(hgroup, "hgroup")
 GK_ATOM(hidden, "hidden")
 GK_ATOM(hidechrome, "hidechrome")
+GK_ATOM(high, "high")
 GK_ATOM(highest, "highest")
 GK_ATOM(horizontal, "horizontal")
 GK_ATOM(hover, "hover")
@@ -426,10 +439,9 @@ GK_ATOM(href, "href")
 GK_ATOM(hreflang, "hreflang")
 GK_ATOM(hspace, "hspace")
 GK_ATOM(html, "html")
-GK_ATOM(htmlBaseHref, "html-base-href")
-GK_ATOM(htmlBaseTarget, "html-base-target")
 GK_ATOM(httpEquiv, "http-equiv")
 GK_ATOM(i, "i")
+GK_ATOM(icon, "icon")
 GK_ATOM(id, "id")
 GK_ATOM(_if, "if")
 GK_ATOM(iframe, "iframe")
@@ -467,6 +479,11 @@ GK_ATOM(iscontainer, "iscontainer")
 GK_ATOM(isempty, "isempty")
 GK_ATOM(isindex, "isindex")
 GK_ATOM(ismap, "ismap")
+GK_ATOM(itemid, "itemid")
+GK_ATOM(itemprop, "itemprop")
+GK_ATOM(itemref, "itemref")
+GK_ATOM(itemscope, "itemscope")
+GK_ATOM(itemtype, "itemtype")
 GK_ATOM(kbd, "kbd")
 GK_ATOM(noautofocus, "noautofocus")
 GK_ATOM(keepcurrentinview, "keepcurrentinview")
@@ -477,6 +494,7 @@ GK_ATOM(keypress, "keypress")
 GK_ATOM(keyset, "keyset")
 GK_ATOM(keytext, "keytext")
 GK_ATOM(keyup, "keyup")
+GK_ATOM(kind, "kind")
 GK_ATOM(label, "label")
 GK_ATOM(lang, "lang")
 GK_ATOM(last, "last")
@@ -494,6 +512,7 @@ GK_ATOM(level, "level")
 GK_ATOM(li, "li")
 GK_ATOM(line, "line")
 GK_ATOM(link, "link")
+GK_ATOM(list, "list")
 GK_ATOM(listbox, "listbox")
 GK_ATOM(listboxbody, "listboxbody")
 GK_ATOM(listcell, "listcell")
@@ -510,9 +529,11 @@ GK_ATOM(localedir, "localedir")
 GK_ATOM(localName, "local-name")
 GK_ATOM(longdesc, "longdesc")
 #ifdef MOZ_MEDIA
+GK_ATOM(loop, "loop")
 GK_ATOM(loopend, "loopend")
 GK_ATOM(loopstart, "loopstart")
 #endif
+GK_ATOM(low, "low")
 GK_ATOM(lowerFirst, "lower-first")
 GK_ATOM(lowest, "lowest")
 GK_ATOM(lowsrc, "lowsrc")
@@ -523,6 +544,7 @@ GK_ATOM(map, "map")
 GK_ATOM(manifest, "manifest")
 GK_ATOM(marginheight, "marginheight")
 GK_ATOM(marginwidth, "marginwidth")
+GK_ATOM(mark, "mark")
 GK_ATOM(marquee, "marquee")
 GK_ATOM(match, "match")
 GK_ATOM(max, "max")
@@ -538,12 +560,14 @@ GK_ATOM(member, "member")
 GK_ATOM(menu, "menu")
 GK_ATOM(menubar, "menubar")
 GK_ATOM(menubutton, "menubutton")
+GK_ATOM(menuButton, "menu-button")
 GK_ATOM(menugenerated, "menugenerated")
 GK_ATOM(menuitem, "menuitem")
 GK_ATOM(menulist, "menulist")
 GK_ATOM(menupopup, "menupopup")
 GK_ATOM(message, "message")
 GK_ATOM(meta, "meta")
+GK_ATOM(meter, "meter")
 GK_ATOM(method, "method")
 GK_ATOM(middle, "middle")
 GK_ATOM(min, "min")
@@ -563,6 +587,8 @@ GK_ATOM(mouseover, "mouseover")
 GK_ATOM(mousethrough, "mousethrough")
 GK_ATOM(mouseup, "mouseup")
 GK_ATOM(moz_opaque, "moz-opaque")
+GK_ATOM(moz_action_hint, "mozactionhint")
+GK_ATOM(x_moz_errormessage, "x-moz-errormessage")
 GK_ATOM(msthemecompatible, "msthemecompatible")
 GK_ATOM(multicol, "multicol")
 GK_ATOM(multiple, "multiple")
@@ -571,9 +597,11 @@ GK_ATOM(_namespace, "namespace")
 GK_ATOM(namespaceAlias, "namespace-alias")
 GK_ATOM(namespaceUri, "namespace-uri")
 GK_ATOM(NaN, "NaN")
+GK_ATOM(nav, "nav")
 GK_ATOM(negate, "negate")
 GK_ATOM(never, "never")
 GK_ATOM(_new, "new")
+GK_ATOM(newline, "newline")
 GK_ATOM(nextBidi, "NextBidi")
 GK_ATOM(no, "no")
 GK_ATOM(noautohide, "noautohide")
@@ -589,6 +617,7 @@ GK_ATOM(normal, "normal")
 GK_ATOM(normalizeSpace, "normalize-space")
 GK_ATOM(noscript, "noscript")
 GK_ATOM(noshade, "noshade")
+GK_ATOM(novalidate, "novalidate")
 GK_ATOM(_not, "not")
 GK_ATOM(nowrap, "nowrap")
 GK_ATOM(number, "number")
@@ -602,9 +631,16 @@ GK_ATOM(OFF, "OFF")
 GK_ATOM(ol, "ol")
 GK_ATOM(omitXmlDeclaration, "omit-xml-declaration")
 GK_ATOM(onabort, "onabort")
+GK_ATOM(onafterscriptexecute, "onafterscriptexecute")
+#ifdef MOZ_CSS_ANIMATIONS
+GK_ATOM(onanimationend, "onanimationend")
+GK_ATOM(onanimationiteration, "onanimationiteration")
+GK_ATOM(onanimationstart, "onanimationstart")
+#endif
 GK_ATOM(onbeforecopy, "onbeforecopy")
 GK_ATOM(onbeforecut, "onbeforecut")
 GK_ATOM(onbeforepaste, "onbeforepaste")
+GK_ATOM(onbeforescriptexecute, "onbeforescriptexecute")
 GK_ATOM(onbeforeunload, "onbeforeunload")
 GK_ATOM(onblur, "onblur")
 GK_ATOM(onbroadcast, "onbroadcast")
@@ -645,18 +681,22 @@ GK_ATOM(onfocus, "onfocus")
 GK_ATOM(onget, "onget")
 GK_ATOM(onhashchange, "onhashchange")
 GK_ATOM(oninput, "oninput")
+GK_ATOM(oninvalid, "oninvalid")
 GK_ATOM(onkeydown, "onkeydown")
 GK_ATOM(onkeypress, "onkeypress")
 GK_ATOM(onkeyup, "onkeyup")
 GK_ATOM(onLoad, "onLoad")
 GK_ATOM(onload, "onload")
+GK_ATOM(onpopstate, "onpopstate")
 GK_ATOM(only, "only")               // this one is not an event
 GK_ATOM(onmousedown, "onmousedown")
 GK_ATOM(onmousemove, "onmousemove")
 GK_ATOM(onmouseout, "onmouseout")
 GK_ATOM(onmouseover, "onmouseover")
+GK_ATOM(onMozMouseHittest, "onMozMouseHittest")
 GK_ATOM(onmouseup, "onmouseup")
 GK_ATOM(onMozAfterPaint, "onMozAfterPaint")
+GK_ATOM(onMozBeforePaint, "onMozBeforePaint")
 GK_ATOM(onMozMousePixelScroll, "onMozMousePixelScroll")
 GK_ATOM(onMozScrolledAreaChanged, "onMozScrolledAreaChanged")
 GK_ATOM(ononline, "ononline")
@@ -671,18 +711,22 @@ GK_ATOM(onpopuphidden, "onpopuphidden")
 GK_ATOM(onpopuphiding, "onpopuphiding")
 GK_ATOM(onpopupshowing, "onpopupshowing")
 GK_ATOM(onpopupshown, "onpopupshown")
+GK_ATOM(onreadystatechange, "onreadystatechange")
 GK_ATOM(onRequest, "onRequest")
 GK_ATOM(onreset, "onreset")
+GK_ATOM(onMozBeforeResize, "onMozBeforeResize")
 GK_ATOM(onresize, "onresize")
 GK_ATOM(onscroll, "onscroll")
 GK_ATOM(onselect, "onselect")
 GK_ATOM(onset, "onset")
 GK_ATOM(onsubmit, "onsubmit")
 GK_ATOM(ontext, "ontext")
+GK_ATOM(ontransitionend, "ontransitionend")
 GK_ATOM(onunderflow, "onunderflow")
 GK_ATOM(onunload, "onunload")
 GK_ATOM(open, "open")
 GK_ATOM(optgroup, "optgroup")
+GK_ATOM(optimum, "optimum")
 GK_ATOM(option, "option")
 GK_ATOM(_or, "or")
 GK_ATOM(order, "order")
@@ -710,6 +754,7 @@ GK_ATOM(param, "param")
 GK_ATOM(parameter, "parameter")
 GK_ATOM(parent, "parent")
 GK_ATOM(parsetype, "parsetype")
+GK_ATOM(pattern, "pattern")
 GK_ATOM(patternSeparator, "pattern-separator")
 GK_ATOM(perMille, "per-mille")
 GK_ATOM(percent, "percent")
@@ -719,6 +764,7 @@ GK_ATOM(ping, "ping")
 #ifdef MOZ_MEDIA
 GK_ATOM(pixelratio, "pixelratio")
 #endif
+GK_ATOM(placeholder, "placeholder")
 GK_ATOM(plaintext, "plaintext")
 #ifdef MOZ_MEDIA
 GK_ATOM(playbackrate, "playbackrate")
@@ -748,6 +794,9 @@ GK_ATOM(preceding, "preceding")
 GK_ATOM(precedingSibling, "preceding-sibling")
 GK_ATOM(predicate, "predicate")
 GK_ATOM(prefix, "prefix")
+#ifdef MOZ_MEDIA
+GK_ATOM(preload, "preload")
+#endif
 GK_ATOM(preserve, "preserve")
 GK_ATOM(preserveSpace, "preserve-space")
 GK_ATOM(preventdefault, "preventdefault")
@@ -756,6 +805,7 @@ GK_ATOM(print, "print")
 GK_ATOM(priority, "priority")
 GK_ATOM(processingInstruction, "processing-instruction")
 GK_ATOM(profile, "profile")
+GK_ATOM(progress, "progress")
 GK_ATOM(progressmeter, "progressmeter")
 GK_ATOM(progressNormal, "progressNormal")
 GK_ATOM(progressUndetermined, "progressUndetermined")
@@ -764,6 +814,8 @@ GK_ATOM(prompt, "prompt")
 GK_ATOM(propagate, "propagate")
 GK_ATOM(properties, "properties")
 GK_ATOM(property, "property")
+GK_ATOM(pubdate, "pubdate")
+GK_ATOM(pushedFloatsList, "PushedFloats-list")
 GK_ATOM(q, "q")
 GK_ATOM(query, "query")
 GK_ATOM(queryset, "queryset")
@@ -778,8 +830,10 @@ GK_ATOM(refresh, "refresh")
 GK_ATOM(rel, "rel")
 GK_ATOM(rem, "rem")
 GK_ATOM(removeelement, "removeelement")
+GK_ATOM(renderingobserverlist, "renderingobserverlist")
 GK_ATOM(repeat, "repeat")
 GK_ATOM(replace, "replace")
+GK_ATOM(required, "required")
 GK_ATOM(reset, "reset")
 GK_ATOM(resizeafter, "resizeafter")
 GK_ATOM(resizebefore, "resizebefore")
@@ -791,6 +845,8 @@ GK_ATOM(result, "result")
 GK_ATOM(resultPrefix, "result-prefix")
 GK_ATOM(rev, "rev")
 GK_ATOM(reverse, "reverse")
+GK_ATOM(reversed, "reversed")
+GK_ATOM(richlistbox, "richlistbox")
 GK_ATOM(right, "right")
 GK_ATOM(rightmargin, "rightmargin")
 GK_ATOM(rightpadding, "rightpadding")
@@ -799,7 +855,10 @@ GK_ATOM(round, "round")
 GK_ATOM(row, "row")
 GK_ATOM(rows, "rows")
 GK_ATOM(rowspan, "rowspan")
+GK_ATOM(rp, "rp")
+GK_ATOM(rt, "rt")
 GK_ATOM(rtl, "rtl")
+GK_ATOM(ruby, "ruby")
 GK_ATOM(rule, "rule")
 GK_ATOM(rules, "rules")
 GK_ATOM(s, "s")
@@ -808,6 +867,7 @@ GK_ATOM(scale, "scale")
 GK_ATOM(scan, "scan")
 GK_ATOM(scheme, "scheme")
 GK_ATOM(scope, "scope")
+GK_ATOM(scoped, "scoped")
 GK_ATOM(screen, "screen")
 GK_ATOM(screenX, "screenX")
 GK_ATOM(screenY, "screenY")
@@ -818,6 +878,7 @@ GK_ATOM(scrollbarbutton, "scrollbarbutton")
 GK_ATOM(scrollbox, "scrollbox")
 GK_ATOM(scrollcorner, "scrollcorner")
 GK_ATOM(scrolling, "scrolling")
+GK_ATOM(section, "section")
 GK_ATOM(select, "select")
 GK_ATOM(selectPopupList, "selectPopupList")
 GK_ATOM(selectable, "selectable")
@@ -831,6 +892,7 @@ GK_ATOM(setter, "setter")
 GK_ATOM(shape, "shape")
 GK_ATOM(show, "show")
 GK_ATOM(showcaret, "showcaret")
+GK_ATOM(showresizer, "showresizer")
 GK_ATOM(simple, "simple")
 GK_ATOM(single, "single")
 GK_ATOM(size, "size")
@@ -844,6 +906,7 @@ GK_ATOM(sort, "sort")
 GK_ATOM(sortActive, "sortActive")
 GK_ATOM(sortDirection, "sortDirection")
 GK_ATOM(sorted, "sorted")
+GK_ATOM(sorthints, "sorthints")
 GK_ATOM(sortLocked, "sortLocked")
 GK_ATOM(sortResource, "sortResource")
 GK_ATOM(sortResource2, "sortResource2")
@@ -858,9 +921,11 @@ GK_ATOM(span, "span")
 GK_ATOM(spellcheck, "spellcheck")
 GK_ATOM(spinner, "spinner")
 GK_ATOM(split, "split")
+GK_ATOM(splitmenu, "splitmenu")
 GK_ATOM(splitter, "splitter")
 GK_ATOM(spring, "spring")
 GK_ATOM(src, "src")
+GK_ATOM(srclang, "srclang")
 GK_ATOM(stack, "stack")
 GK_ATOM(standalone, "standalone")
 GK_ATOM(standby, "standby")
@@ -905,6 +970,7 @@ GK_ATOM(targets, "targets")
 GK_ATOM(tbody, "tbody")
 GK_ATOM(td, "td")
 GK_ATOM(_template, "template")
+GK_ATOM(text_decoration, "text-decoration")
 GK_ATOM(terminate, "terminate")
 GK_ATOM(test, "test")
 GK_ATOM(text, "text")
@@ -915,6 +981,7 @@ GK_ATOM(tfoot, "tfoot")
 GK_ATOM(th, "th")
 GK_ATOM(thead, "thead")
 GK_ATOM(thumb, "thumb")
+GK_ATOM(time, "time")
 GK_ATOM(title, "title")
 GK_ATOM(titlebar, "titlebar")
 GK_ATOM(titletip, "titletip")
@@ -933,6 +1000,7 @@ GK_ATOM(topmargin, "topmargin")
 GK_ATOM(toppadding, "toppadding")
 GK_ATOM(topright, "topright")
 GK_ATOM(tr, "tr")
+GK_ATOM(track, "track")
 GK_ATOM(trailing, "trailing")
 GK_ATOM(transform, "transform")
 GK_ATOM(transformiix, "transformiix")
@@ -1006,6 +1074,9 @@ GK_ATOM(withParam, "with-param")
 GK_ATOM(wizard, "wizard")
 GK_ATOM(wrap, "wrap")
 GK_ATOM(headerDNSPrefetchControl,"x-dns-prefetch-control")
+GK_ATOM(headerCSP, "x-content-security-policy")
+GK_ATOM(headerCSPReportOnly, "x-content-security-policy-report-only")
+GK_ATOM(headerXFO, "x-frame-options")
 GK_ATOM(xml, "xml")
 GK_ATOM(xmlns, "xmlns")
 GK_ATOM(xmp, "xmp")
@@ -1162,7 +1233,7 @@ GK_ATOM(maskContentUnits, "maskContentUnits")
 GK_ATOM(maskUnits, "maskUnits")
 GK_ATOM(matrix, "matrix")
 GK_ATOM(metadata, "metadata")
-GK_ATOM(missingGlyph, "missingGlyph")
+GK_ATOM(missingGlyph, "missing-glyph")
 GK_ATOM(mm, "mm")
 GK_ATOM(mpath, "mpath")
 GK_ATOM(noStitch, "noStitch")
@@ -1182,10 +1253,10 @@ GK_ATOM(opacity, "opacity")
 GK_ATOM(_operator, "operator")
 GK_ATOM(out, "out")
 GK_ATOM(over, "over")
+GK_ATOM(overridePreserveAspectRatio, "overridePreserveAspectRatio")
 GK_ATOM(pad, "pad")
 GK_ATOM(path, "path")
 GK_ATOM(pathLength, "pathLength")
-GK_ATOM(pattern, "pattern")
 GK_ATOM(patternContentUnits, "patternContentUnits")
 GK_ATOM(patternTransform, "patternTransform")
 GK_ATOM(patternUnits, "patternUnits")
@@ -1250,7 +1321,6 @@ GK_ATOM(tableValues, "tableValues")
 GK_ATOM(targetX, "targetX")
 GK_ATOM(targetY, "targetY")
 GK_ATOM(text_anchor, "text-anchor")
-GK_ATOM(text_decoration, "text-decoration")
 GK_ATOM(text_rendering, "text-rendering")
 GK_ATOM(textPath, "textPath")
 GK_ATOM(tref, "tref")
@@ -1279,46 +1349,86 @@ GK_ATOM(accumulate, "accumulate")
 GK_ATOM(additive, "additive")
 GK_ATOM(attributeName, "attributeName")
 GK_ATOM(attributeType, "attributeType")
+GK_ATOM(auto_reverse, "auto-reverse")
 GK_ATOM(begin, "begin")
 GK_ATOM(by, "by")
 GK_ATOM(calcMode, "calcMode")
 GK_ATOM(css, "CSS")
 GK_ATOM(dur, "dur")
+GK_ATOM(keyPoints, "keyPoints")
 GK_ATOM(keySplines, "keySplines")
 GK_ATOM(keyTimes, "keyTimes")
+GK_ATOM(mozAnimateMotionDummyAttr, "_mozAnimateMotionDummyAttr")
+GK_ATOM(onbegin, "onbegin")
+GK_ATOM(onbeginEvent, "onbeginEvent")
+GK_ATOM(onend, "onend")
+GK_ATOM(onendEvent, "onendEvent")
+GK_ATOM(onrepeat, "onrepeat")
+GK_ATOM(onrepeatEvent, "onrepeatEvent")
 GK_ATOM(repeatCount, "repeatCount")
 GK_ATOM(repeatDur, "repeatDur")
+GK_ATOM(repeatEvent, "repeatEvent")
 GK_ATOM(restart, "restart")
 GK_ATOM(to, "to")
 GK_ATOM(XML, "XML")
 #endif
 
 #ifdef MOZ_MATHML
-GK_ATOM(MOZcolumnalign, "-moz-math-columnalign")
-GK_ATOM(MOZcolumnline, "-moz-math-columnline") // different from columnlines_
-GK_ATOM(MOZfontsize, "-moz-math-font-size") // different from fontsize_
-GK_ATOM(MOZfontstyle, "-moz-math-font-style") // different from fontstyle_
-GK_ATOM(MOZrowalign, "-moz-math-rowalign")
-GK_ATOM(MOZrowline, "-moz-math-rowline") // different from rowlines_
+// internal MathML attributes: different from columnalign_, columnlines_,
+// fontstyle_, rowalign_ and rowlines_
+GK_ATOM(_moz_math_columnalign_, "_moz-math-columnalign")
+GK_ATOM(_moz_math_columnline_, "_moz-math-columnline")
+GK_ATOM(_moz_math_fontstyle_, "_moz-math-font-style")
+GK_ATOM(_moz_math_rowalign_, "_moz-math-rowalign")
+GK_ATOM(_moz_math_rowline_, "_moz-math-rowline")
+
 GK_ATOM(abs_, "abs")
 GK_ATOM(accent_, "accent")
 GK_ATOM(accentunder_, "accentunder")
 GK_ATOM(actiontype_, "actiontype")
 GK_ATOM(alignmentscope_, "alignmentscope")
+GK_ATOM(altimg_, "altimg")
+GK_ATOM(altimg_height_, "altimg-height")
+GK_ATOM(altimg_valign_, "altimg-valign")
+GK_ATOM(altimg_width_, "altimg-width")
 GK_ATOM(annotation_, "annotation")
+GK_ATOM(annotation_xml_, "annotation-xml")
 GK_ATOM(apply_, "apply")
+GK_ATOM(approx_, "approx")
 GK_ATOM(arccos_, "arccos")
+GK_ATOM(arccosh_, "arccosh")
+GK_ATOM(arccot_, "arccot")
+GK_ATOM(arccoth_, "arccoth")
+GK_ATOM(arccsc_, "arccsc")
+GK_ATOM(arccsch_, "arccsch")
+GK_ATOM(arcsec_, "arcsec")
+GK_ATOM(arcsech_, "arcsech")
 GK_ATOM(arcsin_, "arcsin")
+GK_ATOM(arcsinh_, "arcsinh")
 GK_ATOM(arctan_, "arctan")
+GK_ATOM(arctanh_, "arctanh")
+GK_ATOM(arg_, "arg")
 GK_ATOM(bevelled_, "bevelled")
+GK_ATOM(bind_, "bind")
 GK_ATOM(bvar_, "bvar")
+GK_ATOM(card_, "card")
+GK_ATOM(cartesianproduct_, "cartesianproduct")
+GK_ATOM(cbytes_, "cbytes")
+GK_ATOM(cd_, "cd")
+GK_ATOM(cdgroup_, "cdgroup")
+GK_ATOM(cerror_, "cerror")
+GK_ATOM(charalign_, "charalign")
 GK_ATOM(ci_, "ci")
+GK_ATOM(closure_, "closure")
 GK_ATOM(cn_, "cn")
+GK_ATOM(codomain_, "codomain")
 GK_ATOM(columnalign_, "columnalign")
+GK_ATOM(columnalignment_, "columnalignment")
 GK_ATOM(columnlines_, "columnlines")
 GK_ATOM(columnspacing_, "columnspacing")
 GK_ATOM(columnspan_, "columnspan")
 GK_ATOM(columnwidth_, "columnwidth")
+GK_ATOM(complexes_, "complexes")
 GK_ATOM(compose_, "compose")
 GK_ATOM(condition_, "condition")
 GK_ATOM(conjugate_, "conjugate")
@@ -1326,22 +1436,37 @@ GK_ATOM(cos_, "cos")
 GK_ATOM(cosh_, "cosh")
 GK_ATOM(cot_, "cot")
 GK_ATOM(coth_, "coth")
+GK_ATOM(crossout_, "crossout")
 GK_ATOM(csc_, "csc")
 GK_ATOM(csch_, "csch")
+GK_ATOM(cs_, "cs")
+GK_ATOM(csymbol_, "csymbol")
+GK_ATOM(curl_, "curl")
+GK_ATOM(decimalpoint_, "decimalpoint")
+GK_ATOM(definitionURL_, "definitionURL")
 GK_ATOM(degree_, "degree")
 GK_ATOM(denomalign_, "denomalign")
 GK_ATOM(depth_, "depth")
 GK_ATOM(determinant_, "determinant")
 GK_ATOM(diff_, "diff")
 GK_ATOM(displaystyle_, "displaystyle")
+GK_ATOM(divergence_, "divergence")
 GK_ATOM(divide_, "divide")
+GK_ATOM(domain_, "domain")
+GK_ATOM(domainofapplication_, "domainofapplication")
 GK_ATOM(edge_, "edge")
+GK_ATOM(el_, "el")
+GK_ATOM(emptyset_, "emptyset")
 GK_ATOM(eq_, "eq")
 GK_ATOM(equalcolumns_, "equalcolumns")
 GK_ATOM(equalrows_, "equalrows")
+GK_ATOM(equivalent_, "equivalent")
+GK_ATOM(eulergamma_, "eulergamma")
 GK_ATOM(exists_, "exists")
 GK_ATOM(exp_, "exp")
+GK_ATOM(exponentiale_, "exponentiale")
 GK_ATOM(factorial_, "factorial")
+GK_ATOM(factorof_, "factorof")
 GK_ATOM(fence_, "fence")
 GK_ATOM(fn_, "fn")
 GK_ATOM(fontfamily_, "fontfamily")
@@ -1350,32 +1475,50 @@ GK_ATOM(fontstyle_, "fontstyle")
 GK_ATOM(fontweight_, "fontweight")
 GK_ATOM(forall_, "forall")
 GK_ATOM(framespacing_, "framespacing")
+GK_ATOM(gcd_, "gcd")
 GK_ATOM(geq_, "geq")
 GK_ATOM(groupalign_, "groupalign")
 GK_ATOM(gt_, "gt")
 GK_ATOM(ident_, "ident")
+GK_ATOM(imaginaryi_, "imaginaryi")
+GK_ATOM(imaginary_, "imaginary")
 GK_ATOM(implies_, "implies")
-GK_ATOM(int_, "int")
+GK_ATOM(indentalignfirst_, "indentalignfirst")
+GK_ATOM(indentalign_, "indentalign")
+GK_ATOM(indentalignlast_, "indentalignlast")
+GK_ATOM(indentshiftfirst_, "indentshiftfirst")
+GK_ATOM(indentshift_, "indentshift")
+GK_ATOM(indenttarget_, "indenttarget")
+GK_ATOM(integers_, "integers")
 GK_ATOM(intersect_, "intersect")
 GK_ATOM(interval_, "interval")
+GK_ATOM(int_, "int")
 GK_ATOM(inverse_, "inverse")
 GK_ATOM(lambda_, "lambda")
+GK_ATOM(laplacian_, "laplacian")
 GK_ATOM(largeop_, "largeop")
+GK_ATOM(lcm_, "lcm")
 GK_ATOM(leq_, "leq")
 GK_ATOM(limit_, "limit")
 GK_ATOM(linebreak_, "linebreak")
+GK_ATOM(linebreakmultchar_, "linebreakmultchar")
+GK_ATOM(linebreakstyle_, "linebreakstyle")
 GK_ATOM(linethickness_, "linethickness")
 GK_ATOM(list_, "list")
 GK_ATOM(ln_, "ln")
-GK_ATOM(log_, "log")
+GK_ATOM(location_, "location")
 GK_ATOM(logbase_, "logbase")
+GK_ATOM(log_, "log")
+GK_ATOM(longdivstyle_, "longdivstyle")
 GK_ATOM(lowlimit_, "lowlimit")
 GK_ATOM(lquote_, "lquote")
 GK_ATOM(lspace_, "lspace")
 GK_ATOM(lt_, "lt")
 GK_ATOM(maction_, "maction")
 GK_ATOM(maligngroup_, "maligngroup")
+GK_ATOM(malign_, "malign")
 GK_ATOM(malignmark_, "malignmark")
+GK_ATOM(malignscope_, "malignscope")
 GK_ATOM(mathbackground_, "mathbackground")
 GK_ATOM(mathcolor_, "mathcolor")
 GK_ATOM(mathsize_, "mathsize")
@@ -1389,15 +1532,20 @@ GK_ATOM(menclose_, "menclose")
 GK_ATOM(merror_, "merror")
 GK_ATOM(mfenced_, "mfenced")
 GK_ATOM(mfrac_, "mfrac")
+GK_ATOM(mfraction_, "mfraction")
+GK_ATOM(mglyph_, "mglyph")
 GK_ATOM(mi_, "mi")
 GK_ATOM(minlabelspacing_, "minlabelspacing")
 GK_ATOM(minsize_, "minsize")
 GK_ATOM(minus_, "minus")
 GK_ATOM(mlabeledtr_, "mlabeledtr")
+GK_ATOM(mlongdiv_, "mlongdiv")
 GK_ATOM(mmultiscripts_, "mmultiscripts")
 GK_ATOM(mn_, "mn")
-GK_ATOM(mo_, "mo")
+GK_ATOM(momentabout_, "momentabout")
 GK_ATOM(moment_, "moment")
+GK_ATOM(mo_, "mo")
+GK_ATOM(monospaced_, "monospaced")
 GK_ATOM(movablelimits_, "movablelimits")
 GK_ATOM(mover_, "mover")
 GK_ATOM(mpadded_, "mpadded")
@@ -1405,9 +1553,15 @@ GK_ATOM(mphantom_, "mphantom")
 GK_ATOM(mprescripts_, "mprescripts")
 GK_ATOM(mroot_, "mroot")
 GK_ATOM(mrow_, "mrow")
+GK_ATOM(mscarries_, "mscarries")
+GK_ATOM(mscarry_, "mscarry")
+GK_ATOM(msgroup_, "msgroup")
+GK_ATOM(msline_, "msline")
 GK_ATOM(ms_, "ms")
 GK_ATOM(mspace_, "mspace")
 GK_ATOM(msqrt_, "msqrt")
+GK_ATOM(msrow_, "msrow")
+GK_ATOM(mstack_, "mstack")
 GK_ATOM(mstyle_, "mstyle")
 GK_ATOM(msub_, "msub")
 GK_ATOM(msubsup_, "msubsup")
@@ -1418,19 +1572,37 @@ GK_ATOM(mtext_, "mtext")
 GK_ATOM(mtr_, "mtr")
 GK_ATOM(munder_, "munder")
 GK_ATOM(munderover_, "munderover")
+GK_ATOM(naturalnumbers_, "naturalnumbers")
+GK_ATOM(negativemediummathspace_, "negativemediummathspace")
+GK_ATOM(negativethickmathspace_, "negativethickmathspace")
+GK_ATOM(negativethinmathspace_, "negativethinmathspace")
+GK_ATOM(negativeverythickmathspace_, "negativeverythickmathspace")
+GK_ATOM(negativeverythinmathspace_, "negativeverythinmathspace")
+GK_ATOM(negativeveryverythickmathspace_, "negativeveryverythickmathspace")
+GK_ATOM(negativeveryverythinmathspace_, "negativeveryverythinmathspace")
 GK_ATOM(neq_, "neq")
+GK_ATOM(notanumber_, "notanumber")
 GK_ATOM(notation_, "notation")
+GK_ATOM(note_, "note")
 GK_ATOM(notin_, "notin")
 GK_ATOM(notprsubset_, "notprsubset")
 GK_ATOM(notsubset_, "notsubset")
 GK_ATOM(numalign_, "numalign")
 GK_ATOM(other_, "other")
+GK_ATOM(outerproduct_, "outerproduct")
 GK_ATOM(partialdiff_, "partialdiff")
+GK_ATOM(piece_, "piece")
+GK_ATOM(piecewise_, "piecewise")
+GK_ATOM(pi_, "pi")
 GK_ATOM(plus_, "plus")
 GK_ATOM(power_, "power")
+GK_ATOM(primes_, "primes")
 GK_ATOM(product_, "product")
 GK_ATOM(prsubset_, "prsubset")
 GK_ATOM(quotient_, "quotient")
+GK_ATOM(rationals_, "rationals")
+GK_ATOM(real_, "real")
+GK_ATOM(reals_, "reals")
 GK_ATOM(reln_, "reln")
 GK_ATOM(root_, "root")
 GK_ATOM(rowalign_, "rowalign")
@@ -1438,29 +1610,36 @@ GK_ATOM(rowlines_, "rowlines")
 GK_ATOM(rowspacing_, "rowspacing")
 GK_ATOM(rquote_, "rquote")
 GK_ATOM(rspace_, "rspace")
+GK_ATOM(scalarproduct_, "scalarproduct")
+GK_ATOM(schemaLocation_, "schemaLocation")
 GK_ATOM(scriptlevel_, "scriptlevel")
 GK_ATOM(scriptminsize_, "scriptminsize")
 GK_ATOM(scriptsizemultiplier_, "scriptsizemultiplier")
+GK_ATOM(scriptsize_, "scriptsize")
 GK_ATOM(sdev_, "sdev")
-GK_ATOM(sec_, "sec")
 GK_ATOM(sech_, "sech")
+GK_ATOM(sec_, "sec")
 GK_ATOM(selection_, "selection")
+GK_ATOM(selector_, "selector")
 GK_ATOM(semantics_, "semantics")
-GK_ATOM(sep_, "sep")
 GK_ATOM(separator_, "separator")
 GK_ATOM(separators_, "separators")
-GK_ATOM(set_, "set")
+GK_ATOM(sep_, "sep")
 GK_ATOM(setdiff_, "setdiff")
+GK_ATOM(set_, "set")
+GK_ATOM(share_, "share")
+GK_ATOM(shift_, "shift")
 GK_ATOM(side_, "side")
-GK_ATOM(sin_, "sin")
 GK_ATOM(sinh_, "sinh")
+GK_ATOM(sin_, "sin")
+GK_ATOM(stackalign_, "stackalign")
 GK_ATOM(stretchy_, "stretchy")
 GK_ATOM(subscriptshift_, "subscriptshift")
 GK_ATOM(subset_, "subset")
 GK_ATOM(superscriptshift_, "superscriptshift")
 GK_ATOM(symmetric_, "symmetric")
-GK_ATOM(tan_, "tan")
 GK_ATOM(tanh_, "tanh")
+GK_ATOM(tan_, "tan")
 GK_ATOM(tendsto_, "tendsto")
 GK_ATOM(thickmathspace_, "thickmathspace")
 GK_ATOM(thinmathspace_, "thinmathspace")
@@ -1468,12 +1647,15 @@ GK_ATOM(times_, "times")
 GK_ATOM(transpose_, "transpose")
 GK_ATOM(union_, "union")
 GK_ATOM(uplimit_, "uplimit")
-GK_ATOM(var_, "var")
+GK_ATOM(variance_, "variance")
+GK_ATOM(vectorproduct_, "vectorproduct")
 GK_ATOM(vector_, "vector")
 GK_ATOM(verythickmathspace_, "verythickmathspace")
 GK_ATOM(verythinmathspace_, "verythinmathspace")
 GK_ATOM(veryverythickmathspace_, "veryverythickmathspace")
 GK_ATOM(veryverythinmathspace_, "veryverythinmathspace")
+GK_ATOM(voffset_, "voffset")
+GK_ATOM(xref_, "xref")
 GK_ATOM(math, "math") // the only one without an underscore
 #endif
 
@@ -1505,6 +1687,11 @@ GK_ATOM(onMozRotateGesture, "onMozRotateGesture")
 GK_ATOM(onMozTapGesture, "onMozTapGesture")
 GK_ATOM(onMozPressTapGesture, "onMozPressTapGesture")
 
+// Touch events
+GK_ATOM(onMozTouchDown, "onMozTouchDown")
+GK_ATOM(onMozTouchMove, "onMozTouchMove")
+GK_ATOM(onMozTouchUp, "onMozTouchUp")
+
 // orientation support
 GK_ATOM(onMozOrientation, "onMozOrientation")
 
@@ -1625,8 +1812,8 @@ GK_ATOM(onended, "onended")
 GK_ATOM(onratechange, "onratechange")
 GK_ATOM(ondurationchange, "ondurationchange")
 GK_ATOM(onvolumechange, "onvolumechange")
+GK_ATOM(onMozAudioAvailable, "onMozAudioAvailable")
 GK_ATOM(loadstart, "loadstart")
-GK_ATOM(progress, "progress")
 GK_ATOM(suspend, "suspend")
 GK_ATOM(emptied, "emptied")
 GK_ATOM(stalled, "stalled")
@@ -1647,40 +1834,15 @@ GK_ATOM(durationchange, "durationchange")
 GK_ATOM(volumechange, "volumechange")
 #endif
 
-// Frame property names
-GK_ATOM(boxMetricsProperty, "BoxMetricsProperty") // nsBoxLayoutMetrics*
-GK_ATOM(changeListProperty, "ChangeListProperty") // void*
-GK_ATOM(collapseOffsetProperty, "CollapseOffsetProperty")  // nsPoint*
-GK_ATOM(computedOffsetProperty, "ComputedOffsetProperty")  // nsPoint*
-GK_ATOM(floatRegionProperty, "FloatRegionProperty") // nsRect*
-GK_ATOM(generatedContent, "GeneratedContentProperty")  // nsCOMArray<nsIContent>*
-#ifdef MOZ_MATHML
-GK_ATOM(HTMLReflowMetricsProperty, "HTMLReflowMetricsProperty") // nsHTMLReflowMetrics*
-#endif
-GK_ATOM(IBSplitSpecialPrevSibling, "IBSplitSpecialPrevSibling")// nsIFrame*
-GK_ATOM(IBSplitSpecialSibling, "IBSplitSpecialSibling")    // nsIFrame*
-GK_ATOM(lineCursorProperty, "LineCursorProperty") // nsLineBox*
-GK_ATOM(rowCursorProperty, "RowCursorProperty") // nsTableRowGroupFrame::FrameCursorData*
-GK_ATOM(maxElementWidthProperty, "MaxElementWidthProperty")  // nscoord*
-GK_ATOM(outlineInnerRectProperty, "OutlineInnerRectProperty") // nsRect*
-GK_ATOM(outOfFlowDirtyRectProperty, "OutOfFlowDirtyRectProperty") // nsRect*
-GK_ATOM(overflowAreaProperty, "OverflowArea")              // nsRect*
-GK_ATOM(overflowProperty, "OverflowProperty")              // list of nsIFrame*
-GK_ATOM(overflowContainersProperty, "OverflowContainersProperty")             // nsFrameList*
-GK_ATOM(excessOverflowContainersProperty, "ExcessOverflowContainersProperty") // nsFrameList*
-GK_ATOM(overflowLinesProperty, "OverflowLinesProperty")    // list of nsLineBox*
-GK_ATOM(overflowOutOfFlowsProperty, "OverflowOutOfFlowsProperty")      // nsFrameList*
-GK_ATOM(overflowPlaceholdersProperty, "OverflowPlaceholdersProperty")  // nsFrameList*
-GK_ATOM(preEffectsBBoxProperty, "PreEffectsBBoxProperty") // nsRect*
-GK_ATOM(preTransformBBoxProperty, "PreTransformBBoxProperty") // nsRect*
-GK_ATOM(rowUnpaginatedHeightProperty, "RowUnpaginatedHeightProperty")  // nscoord*
-GK_ATOM(tabWidthProperty, "TabWidthProperty")              // nsTArray<TabSetting>* array of tab widths
-GK_ATOM(tableBCProperty, "TableBCProperty")                // table border collapsing info (e.g. damage area, table border widths)
-GK_ATOM(usedMarginProperty, "UsedMarginProperty") // nsMargin*
-GK_ATOM(usedPaddingProperty, "UsedPaddingProperty") // nsMargin*
-GK_ATOM(viewProperty, "ViewProperty")                      
-
 // Content property names
+#ifdef MOZ_CSS_ANIMATIONS
+GK_ATOM(animationsProperty, "AnimationsProperty")        // FrameAnimations*
+GK_ATOM(animationsOfBeforeProperty, "AnimationsOfBeforeProperty") // FrameAnimations*
+GK_ATOM(animationsOfAfterProperty, "AnimationsOfAfterProperty") // FrameAnimations*
+#endif
+GK_ATOM(transitionsProperty, "TransitionsProperty")        // FrameTransitions*
+GK_ATOM(transitionsOfBeforeProperty, "TransitionsOfBeforeProperty") // FrameTransitions*
+GK_ATOM(transitionsOfAfterProperty, "TransitionsOfAfterProperty") // FrameTransitions*
 GK_ATOM(genConInitializerProperty, "QuoteNodeProperty")
 GK_ATOM(labelMouseDownPtProperty, "LabelMouseDownPtProperty")
 
@@ -1696,6 +1858,11 @@ GK_ATOM(TypingTxnName, "Typing")
 GK_ATOM(IMETxnName, "IME")
 GK_ATOM(DeleteTxnName, "Deleting")
 
+// IPC stuff
+GK_ATOM(Remote, "remote")
+GK_ATOM(RemoteId, "_remote_id")
+GK_ATOM(DisplayPort, "_displayport")
+
 // Names for system metrics
 GK_ATOM(scrollbar_start_backward, "scrollbar-start-backward")
 GK_ATOM(scrollbar_start_forward, "scrollbar-start-forward")
@@ -1703,12 +1870,23 @@ GK_ATOM(scrollbar_end_backward, "scrollbar-end-backward")
 GK_ATOM(scrollbar_end_forward, "scrollbar-end-forward")
 GK_ATOM(scrollbar_thumb_proportional, "scrollbar-thumb-proportional")
 GK_ATOM(images_in_menus, "images-in-menus")
+GK_ATOM(images_in_buttons, "images-in-buttons")
 GK_ATOM(windows_default_theme, "windows-default-theme")
 GK_ATOM(mac_graphite_theme, "mac-graphite-theme")
 GK_ATOM(windows_compositor, "windows-compositor")
-GK_ATOM(windows_classic, "windows-classic")
 GK_ATOM(touch_enabled, "touch-enabled")
 GK_ATOM(maemo_classic, "maemo-classic")
+GK_ATOM(menubar_drag, "menubar-drag")
+
+// windows theme selector metrics
+GK_ATOM(windows_classic, "windows-classic")
+GK_ATOM(windows_theme_aero, "windows-theme-aero")
+GK_ATOM(windows_theme_luna_blue, "windows-theme-luna-blue")
+GK_ATOM(windows_theme_luna_olive, "windows-theme-luna-olive")
+GK_ATOM(windows_theme_luna_silver, "windows-theme-luna-silver")
+GK_ATOM(windows_theme_royale, "windows-theme-royale")
+GK_ATOM(windows_theme_zune, "windows-theme-zune")
+GK_ATOM(windows_theme_generic, "windows-theme-generic")
 
 // And the same again, as media query keywords.
 GK_ATOM(_moz_scrollbar_start_backward, "-moz-scrollbar-start-backward")
@@ -1717,9 +1895,14 @@ GK_ATOM(_moz_scrollbar_end_backward, "-moz-scrollbar-end-backward")
 GK_ATOM(_moz_scrollbar_end_forward, "-moz-scrollbar-end-forward")
 GK_ATOM(_moz_scrollbar_thumb_proportional, "-moz-scrollbar-thumb-proportional")
 GK_ATOM(_moz_images_in_menus, "-moz-images-in-menus")
+GK_ATOM(_moz_images_in_buttons, "-moz-images-in-buttons")
 GK_ATOM(_moz_windows_default_theme, "-moz-windows-default-theme")
 GK_ATOM(_moz_mac_graphite_theme, "-moz-mac-graphite-theme")
 GK_ATOM(_moz_windows_compositor, "-moz-windows-compositor")
 GK_ATOM(_moz_windows_classic, "-moz-windows-classic")
+GK_ATOM(_moz_windows_theme, "-moz-windows-theme")
 GK_ATOM(_moz_touch_enabled, "-moz-touch-enabled")
 GK_ATOM(_moz_maemo_classic, "-moz-maemo-classic")
+GK_ATOM(_moz_menubar_drag, "-moz-menubar-drag")
+GK_ATOM(_moz_device_pixel_ratio, "-moz-device-pixel-ratio")
+GK_ATOM(_moz_device_orientation, "-moz-device-orientation")
