@@ -124,18 +124,17 @@ SVGPathData::AppendSeg(PRUint32 aType, ...)
 float
 SVGPathData::GetPathLength() const
 {
-  float length = 0.0;
   SVGPathTraversalState state;
 
   PRUint32 i = 0;
   while (i < mData.Length()) {
-    length += SVGPathSegUtils::GetLength(&mData[i], state);
+    SVGPathSegUtils::TraversePathSegment(&mData[i], state);
     i += 1 + SVGPathSegUtils::ArgCountForType(mData[i]);
   }
 
   NS_ABORT_IF_FALSE(i == mData.Length(), "Very, very bad - mData corrupt");
 
-  return length;
+  return state.length;
 }
 
 #ifdef DEBUG
@@ -163,7 +162,9 @@ SVGPathData::GetSegmentLengths(nsTArray<double> *aLengths) const
 
   PRUint32 i = 0;
   while (i < mData.Length()) {
-    if (!aLengths->AppendElement(SVGPathSegUtils::GetLength(&mData[i], state))) {
+    state.length = 0.0;
+    SVGPathSegUtils::TraversePathSegment(&mData[i], state);
+    if (!aLengths->AppendElement(state.length)) {
       aLengths->Clear();
       return PR_FALSE;
     }
@@ -178,7 +179,6 @@ SVGPathData::GetSegmentLengths(nsTArray<double> *aLengths) const
 PRBool
 SVGPathData::GetDistancesFromOriginToEndsOfVisibleSegments(nsTArray<double> *aOutput) const
 {
-  double distRunningTotal = 0.0;
   SVGPathTraversalState state;
 
   aOutput->Clear();
@@ -186,6 +186,7 @@ SVGPathData::GetDistancesFromOriginToEndsOfVisibleSegments(nsTArray<double> *aOu
   PRUint32 i = 0;
   while (i < mData.Length()) {
     PRUint32 segType = SVGPathSegUtils::DecodeType(mData[i]);
+    SVGPathSegUtils::TraversePathSegment(&mData[i], state);
 
     // We skip all moveto commands except an initial moveto. See the text 'A
     // "move to" command does not count as an additional point when dividing up
@@ -199,8 +200,7 @@ SVGPathData::GetDistancesFromOriginToEndsOfVisibleSegments(nsTArray<double> *aOu
 
     if (i == 0 || (segType != nsIDOMSVGPathSeg::PATHSEG_MOVETO_ABS &&
                    segType != nsIDOMSVGPathSeg::PATHSEG_MOVETO_REL)) {
-      distRunningTotal += SVGPathSegUtils::GetLength(&mData[i], state);
-      if (!aOutput->AppendElement(distRunningTotal)) {
+      if (!aOutput->AppendElement(state.length)) {
         return PR_FALSE;
       }
     }
@@ -220,13 +220,12 @@ SVGPathData::GetPathSegAtLength(float aDistance) const
   // Return -1? Throwing would better help authors avoid tricky bugs (DOM
   // could do that if we return -1).
 
-  double distRunningTotal = 0.0;
   PRUint32 i = 0, segIndex = 0;
   SVGPathTraversalState state;
 
   while (i < mData.Length()) {
-    distRunningTotal += SVGPathSegUtils::GetLength(&mData[i], state);
-    if (distRunningTotal >= aDistance) {
+    SVGPathSegUtils::TraversePathSegment(&mData[i], state);
+    if (state.length >= aDistance) {
       return segIndex;
     }
     i += 1 + SVGPathSegUtils::ArgCountForType(mData[i]);
