@@ -216,6 +216,7 @@ _cairo_truetype_font_create (cairo_scaled_font_subset_t  *scaled_font_subset,
     if (font->base.units_per_em == 0)
         font->base.units_per_em = 2048;
 
+    font->base.ps_name = NULL;
     font->base.font_name = NULL;
     status = _cairo_truetype_read_font_name (scaled_font_subset->scaled_font,
 					     &font->base.ps_name,
@@ -647,7 +648,7 @@ cairo_truetype_font_write_head_table (cairo_truetype_font_t *font,
     if (unlikely (status))
 	return _cairo_truetype_font_set_error (font, status);
 
-    /* set checkSumAdjustment to 0 for table checksum calcualtion */
+    /* set checkSumAdjustment to 0 for table checksum calculation */
     *(uint32_t *)(buffer + 8) = 0;
 
     return CAIRO_STATUS_SUCCESS;
