@@ -824,3 +824,17 @@ qcms_profile* qcms_profile_from_path(const char *path)
 	}
 	return profile;
 }
+
+#ifdef _WIN32
+/* Unicode path version */
+qcms_profile* qcms_profile_from_unicode_path(const wchar_t *path)
+{
+	qcms_profile *profile = NULL;
+	FILE *file = _wfopen(path, L"rb");
+	if (file) {
+		profile = qcms_profile_from_file(file);
+		fclose(file);
+	}
+	return profile;
+}
+#endif
