@@ -52,7 +52,6 @@
 #include "jstypes.h"
 #include "methodjit/Compiler.h"
 #include "methodjit/StubCalls.h"
-#include "jstracer.h"
 
 #include "jsinterpinlines.h"
 #include "jspropertycache.h"
@@ -420,7 +419,7 @@ stubs::GetElem(VMFrame &f)
         JSString *str = lref.toString();
         int32_t i = rref.toInt32();
         if ((size_t)i < str->length()) {
-            str = JSString::getUnitString(cx, str, (size_t)i);
+            str = JSAtom::getUnitStringForElement(cx, str, (size_t)i);
             if (!str)
                 THROW();
             f.regs.sp[-2].setString(str);
@@ -1179,13 +1178,7 @@ stubs::Debugger(VMFrame &f, jsbytecode *pc)
           case JSTRAP_RETURN:
             f.cx->clearPendingException();
             f.cx->fp()->setReturnValue(rval);
-#if (defined(JS_NO_FASTCALL) && defined(JS_CPU_X86)) || defined(_WIN64)
-            *f.returnAddressLocation() = JS_FUNC_TO_DATA_PTR(void *,
-                                         f.cx->jaegerCompartment()->forceReturnFastTrampoline());
-#else
-            *f.returnAddressLocation() = JS_FUNC_TO_DATA_PTR(void *,
-                                         f.cx->jaegerCompartment()->forceReturnTrampoline());
-#endif
+            *f.returnAddressLocation() = f.cx->jaegerCompartment()->forceReturnFromFastCall();
             break;
 
           case JSTRAP_ERROR:
@@ -1239,13 +1232,7 @@ stubs::Trap(VMFrame &f, uint32 trapTypes)
       case JSTRAP_RETURN:
         f.cx->clearPendingException();
         f.cx->fp()->setReturnValue(rval);
-#if (defined(JS_NO_FASTCALL) && defined(JS_CPU_X86)) || defined(_WIN64)
-        *f.returnAddressLocation() = JS_FUNC_TO_DATA_PTR(void *,
-                                     f.cx->jaegerCompartment()->forceReturnFastTrampoline());
-#else
-        *f.returnAddressLocation() = JS_FUNC_TO_DATA_PTR(void *,
-                                     f.cx->jaegerCompartment()->forceReturnTrampoline());
-#endif
+        *f.returnAddressLocation() = f.cx->jaegerCompartment()->forceReturnFromFastCall();
         break;
 
       case JSTRAP_ERROR:
@@ -2003,7 +1990,7 @@ stubs::CallProp(VMFrame &f, JSAtom *origAtom)
     }
 #if JS_HAS_NO_SUCH_METHOD
     if (JS_UNLIKELY(rval.isUndefined()) && regs.sp[-1].isObject()) {
-        regs.sp[-2].setString(ATOM_TO_STRING(origAtom));
+        regs.sp[-2].setString(origAtom);
         if (!js_OnUnknownMethod(cx, regs.sp - 2))
             THROW();
     }
@@ -2174,7 +2161,7 @@ stubs::TypeOf(VMFrame &f)
     const Value &ref = f.regs.sp[-1];
     JSType type = JS_TypeOfValue(f.cx, Jsvalify(ref));
     JSAtom *atom = f.cx->runtime->atomState.typeAtoms[type];
-    return ATOM_TO_STRING(atom);
+    return atom;
 }
 
 void JS_FASTCALL
@@ -2377,7 +2364,7 @@ stubs::LookupSwitch(VMFrame &f, jsbytecode *pc)
             Value rval = script->getConst(GET_INDEX(pc));
             pc += INDEX_LEN;
             if (rval.isString()) {
-                JSLinearString *rhs = rval.toString()->assertIsLinear();
+                JSLinearString *rhs = &rval.toString()->asLinear();
                 if (rhs == str || EqualStrings(str, rhs)) {
                     void* native = script->nativeCodeForPC(ctor,
                                                            jpc + GET_JUMP_OFFSET(pc));
