@@ -19,7 +19,8 @@
  * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Igor Bukanov <igor@mir2.org>
+ *                 Bob Clary <bob@bclary.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,41 +37,34 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-var summary = "13.3.5.2 - root QName.uri";
-var BUGNUMBER = 313952;
-var actual = '';
-var expect = '';
+var summary = 'Uncontrolled recursion in js_MarkXML during GC';
+var BUGNUMBER = 280844;
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
-var str = String(1);
-var expected = String(1);
+var N = 100 * 1000;
 
-var x = new XML("text");
-
-x.function::toString = function() {
-        var tmp = str;
-        str = null;
-        return tmp;
+function prepare_list(N)
+{
+    var head = {};
+    var cursor = head;
+    for (var i = 0; i != N; ++i) {
+        var ns = new Namespace(); 
+        var x = <xml/>;
+        x.addNamespace(ns);
+        cursor.next = x;
+        cursor = ns;
+    }
+    return head;
 }
 
-var TWO = 2.0;
+var head = prepare_list(N);
 
-var likeString = {
-        toString: function() {
-                var tmp = new XML("");
-                tmp = (tmp == "string");
-                if (typeof gc == "function")
-                        gc();
-                for (var i = 0; i != 40000; ++i) {
-                        tmp = 1e100 * TWO;
-                        tmp = null;
-                }
-                return expected;
-        }
-}
+gc();
 
-TEST(1, true, x == likeString);
+TEST(1, expect, actual);
 
 END();
