@@ -38,75 +38,51 @@
 
 
 /**
-   File Name:          15.9.5.23-2.js
-   ECMA Section:       15.9.5.23
-   Description:        Date.prototype.setTime
+   File Name:          15.9.5.22.js
+   ECMA Section:       15.9.5.22
+   Description:        Date.prototype.getTimezoneOffset
 
-   1.  If the this value is not a Date object, generate a runtime error.
-   2.  Call ToNumber(time).
-   3.  Call TimeClip(Result(1)).
-   4.  Set the [[Value]] property of the this value to Result(2).
-   5.  Return the value of the [[Value]] property of the this value.
+   Returns the difference between local time and UTC time in minutes.
+   1.  Let t be this time value.
+   2.  If t is NaN, return NaN.
+   3.  Return (t - LocalTime(t)) / msPerMinute.
 
    Author:             christine@netscape.com
    Date:               12 november 1997
 */
 
-var SECTION = "15.9.5.23-2";
+var SECTION = "15.9.5.22";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Date.prototype.setTime()";
+var TITLE   = "Date.prototype.getTimezoneOffset()";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-test_times = new Array( TIME_NOW, TIME_0000, TIME_1970, TIME_1900, TIME_2000,
-			UTC_FEB_29_2000, UTC_JAN_1_2005 );
-
-
-for ( var j = 0; j < test_times.length; j++ ) {
-  addTestCase( new Date(TIME_1970), test_times[j] );
-}
-
+addTestCase( TIME_0000 );
+addTestCase( TIME_1970 );
+addTestCase( TIME_1900 );
+addTestCase( TIME_2000 );
+addTestCase( UTC_FEB_29_2000 );
+addTestCase( UTC_JAN_1_2005 );
 
 new TestCase( SECTION,
-	      "(new Date(NaN)).setTime()",
+	      "(new Date(NaN)).getTimezoneOffset()",
 	      NaN,
-	      (new Date(NaN)).setTime() );
+	      (new Date(NaN)).getTimezoneOffset() );
 
 new TestCase( SECTION,
-	      "Date.prototype.setTime.length",
-	      1,
-	      Date.prototype.setTime.length );
+	      "Date.prototype.getTimezoneOffset.length",
+	      0,
+	      Date.prototype.getTimezoneOffset.length );
+
 test();
 
-function addTestCase( d, t ) {
-  new TestCase( SECTION,
-		"( "+d+" ).setTime("+t+")",
-		t,
-		d.setTime(t) );
-
-  new TestCase( SECTION,
-		"( "+d+" ).setTime("+(t+1.1)+")",
-		TimeClip(t+1.1),
-		d.setTime(t+1.1) );
-
-  new TestCase( SECTION,
-		"( "+d+" ).setTime("+(t+1)+")",
-		t+1,
-		d.setTime(t+1) );
-
-  new TestCase( SECTION,
-		"( "+d+" ).setTime("+(t-1)+")",
-		t-1,
-		d.setTime(t-1) );
-
-  new TestCase( SECTION,
-		"( "+d+" ).setTime("+(t-TZ_ADJUST)+")",
-		t-TZ_ADJUST,
-		d.setTime(t-TZ_ADJUST) );
-
-  new TestCase( SECTION,
-		"( "+d+" ).setTime("+(t+TZ_ADJUST)+")",
-		t+TZ_ADJUST,
-		d.setTime(t+TZ_ADJUST) );
+function addTestCase( t ) {
+  for ( m = 0; m <= 1000; m+=100 ) {
+    t++;
+    new TestCase( SECTION,
+		  "(new Date("+t+")).getTimezoneOffset()",
+		  (t - LocalTime(t)) / msPerMinute,
+		  (new Date(t)).getTimezoneOffset() );
+  }
 }
