@@ -38,7 +38,7 @@
 
 
 /**
-   File Name:          7.3-4.js
+   File Name:          7.3-5.js
    ECMA Section:       7.3 Comments
    Description:
 
@@ -47,7 +47,7 @@
    Date:               12 november 1997
 
 */
-var SECTION = "7.3-4";
+var SECTION = "7.3-5";
 var VERSION = "ECMA_1";
 startTest();
 var TITLE   = "Comments";
@@ -55,10 +55,10 @@ var TITLE   = "Comments";
 writeHeaderToLog( SECTION + " "+ TITLE);
 
 var testcase = new TestCase( SECTION,
-			     "multiline comment ",
+			     "a comment with a carriage return, and text following",
 			     "pass",
 			     "pass");
 
-/*testcase.actual = "fail";*/
+// "\u000A" testcase.actual = "fail";
 
 test();
