@@ -36,99 +36,34 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '15.8.2.8.js';
 
 /**
-   File Name:          15.8.2.8.js
-   ECMA Section:       15.8.2.8  Math.exp(x)
-   Description:        return an approximation to the exponential function of
-   the argument (e raised to the power of the argument)
-   special cases:
-   -   if x is NaN         return NaN
-   -   if x is 0           return 1
-   -   if x is -0          return 1
-   -   if x is Infinity    return Infinity
-   -   if x is -Infinity   return 0
+   File Name:          15.8.1.5-2.js
+   ECMA Section:       15.8.1.5.js
+   Description:        All value properties of the Math object should have
+   the attributes [DontEnum, DontDelete, ReadOnly]
+
+   this test checks the DontDelete attribute of Math.LOG10E
+
    Author:             christine@netscape.com
-   Date:               7 july 1997
+   Date:               16 september 1997
 */
 
-
-var SECTION = "15.8.2.8";
+var SECTION = "15.8.1.5-2";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Math.exp(x)";
+var TITLE   = "Math.LOG10E";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
 new TestCase( SECTION,
-	      "Math.exp.length",
-	      1,
-	      Math.exp.length );
+	      "delete Math.LOG10E; Math.LOG10E",  
+	      0.4342944819032518,    
+	      eval("delete Math.LOG10E; Math.LOG10E") );
 
 new TestCase( SECTION,
-	      "Math.exp()",
-	      Number.NaN,
-	      Math.exp() );
-
-new TestCase( SECTION,
-	      "Math.exp(null)",
-	      1,
-	      Math.exp(null) );
-
-new TestCase( SECTION,
-	      "Math.exp(void 0)",
-	      Number.NaN,
-	      Math.exp(void 0) );
-
-new TestCase( SECTION,
-	      "Math.exp(1)",
-	      Math.E,
-	      Math.exp(1) );
-
-new TestCase( SECTION,
-	      "Math.exp(true)",
-	      Math.E,
-	      Math.exp(true) );
-
-new TestCase( SECTION,
-	      "Math.exp(false)",
-	      1,
-	      Math.exp(false) );
-
-new TestCase( SECTION,
-	      "Math.exp('1')",
-	      Math.E,
-	      Math.exp('1') );
-
-new TestCase( SECTION,
-	      "Math.exp('0')",
-	      1,
-	      Math.exp('0') );
-
-new TestCase( SECTION,
-	      "Math.exp(NaN)",
-	      Number.NaN,
-	      Math.exp(Number.NaN) );
-
-new TestCase( SECTION,
-	      "Math.exp(0)",
-	      1,
-	      Math.exp(0)          );
-
-new TestCase( SECTION,
-	      "Math.exp(-0)",
-	      1,
-	      Math.exp(-0)         );
-
-new TestCase( SECTION,
-	      "Math.exp(Infinity)",
-	      Number.POSITIVE_INFINITY,
-	      Math.exp(Number.POSITIVE_INFINITY) );
-
-new TestCase( SECTION,
-	      "Math.exp(-Infinity)", 
-	      0,
-	      Math.exp(Number.NEGATIVE_INFINITY) );
+	      "delete Math.LOG10E",               
+	      false,                 
+	      eval("delete Math.LOG10E") );
 
 test();
