@@ -38,31 +38,28 @@
 
 
 /**
-   File Name:          15.7.3.js
-   15.7.3  Properties of the Number Constructor
+   File Name:          15.2.4.js
+   ECMA Section:       15.2.4 Properties of the Object prototype object
 
-   Description:        The value of the internal [[Prototype]] property
-   of the Number constructor is the Function prototype
-   object.  The Number constructor also has the internal
-   [[Call]] and [[Construct]] properties, and the length
-   property.
-
-   Other properties are in subsequent tests.
+   Description:        The value of the internal [[Prototype]] property of
+   the Object prototype object is null
 
    Author:             christine@netscape.com
-   Date:               29 september 1997
+   Date:               28 october 1997
+
 */
 
-var SECTION = "15.7.3";
+var SECTION = "15.2.4";
 var VERSION = "ECMA_2";
 startTest();
-var TITLE   = "Properties of the Number Constructor";
+var TITLE   = "Properties of the Object.prototype object";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-new TestCase(SECTION,
-	     "Number.__proto__",  
-	     Function.prototype,
-	     Number.__proto__ );
+new TestCase( SECTION, 
+	      "Object.prototype.__proto__",
+	      null,
+	      Object.prototype.__proto__ );
 
 test();
+
