@@ -16,10 +16,11 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2007
+ * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Igor Bukanov <igor@mir2.org>
+ *                 Bob Clary <bob@bclary.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,26 +36,34 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 385393;
-var summary = 'Regression test for bug 385393';
-var actual = 'No Crash';
-var expect = 'No Crash';
+var BUGNUMBER = 230216;
+var summary = 'check for numerical overflow in regexps in back reference and bounds for {} quantifier';
+var actual = '';
+var expect = '';
+var status = '';
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
+DESCRIPTION = summary;
+EXPECTED = 'error';
 
-function test()
+printBugNumber(BUGNUMBER);
+printStatus (summary);
+
+status = inSection(1) + ' check for overflow in quantifier';
+
+actual = 'undefined'; 
+expect = 'error';
+
+try
 {
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
- 
-  reportCompare(expect, actual, summary);
-
-  true.watch("x", function(){});
-
-  exitFunc ('test');
+  var result = eval('/a{21474836481}/.test("a")');
+  actual = 'no exception thrown';
+  status += ' result: ' + result;
 }
+catch(e)
+{
+  actual = 'error';
+}
+
+reportCompare(expect, actual, status);
+
