@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s):
+ * Contributor(s): Gary Kwong
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -52,13 +52,14 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  expect = '1';
+// ------- Comment #135 From Gary Kwong [:nth10sd]
 
-// ------- Comment #77 From Brendan Eich
-
-  (function () { const [d] = [1]; [d] = [2]; print(actual = d);})();
-
-  actual = String(actual);
+  if (typeof timeout == 'function')
+  {
+    expectExitCode(6);
+    timeout(3);
+    eval("do ([]); while(y for each (x in []))");
+  }
 
   reportCompare(expect, actual, summary);
 
