@@ -38,13 +38,14 @@
 #include "nsLayoutStylesheetCache.h"
 
 #include "nsAppDirectoryServiceDefs.h"
-#include "nsICSSLoader.h"
+#include "mozilla/css/Loader.h"
 #include "nsIFile.h"
 #include "nsLayoutCID.h"
 #include "nsNetUtil.h"
 #include "nsIObserverService.h"
 #include "nsServiceManagerUtils.h"
 #include "nsIXULRuntime.h"
+#include "nsCSSStyleSheet.h"
 
 NS_IMPL_ISUPPORTS1(nsLayoutStylesheetCache, nsIObserver)
 
@@ -71,7 +72,7 @@ nsLayoutStylesheetCache::Observe(nsISupports* aSubject,
   return NS_OK;
 }
 
-nsICSSStyleSheet*
+nsCSSStyleSheet*
 nsLayoutStylesheetCache::ScrollbarsSheet()
 {
   EnsureGlobal();
@@ -85,14 +86,14 @@ nsLayoutStylesheetCache::ScrollbarsSheet()
 
     // Scrollbars don't need access to unsafe rules
     if (sheetURI)
-      LoadSheet(sheetURI, gStyleCache->mScrollbarsSheet, PR_FALSE, PR_TRUE);
+      LoadSheet(sheetURI, gStyleCache->mScrollbarsSheet, PR_FALSE);
     NS_ASSERTION(gStyleCache->mScrollbarsSheet, "Could not load scrollbars.css.");
   }
 
   return gStyleCache->mScrollbarsSheet;
 }
 
-nsICSSStyleSheet*
+nsCSSStyleSheet*
 nsLayoutStylesheetCache::FormsSheet()
 {
   EnsureGlobal();
@@ -102,11 +103,11 @@ nsLayoutStylesheetCache::FormsSheet()
   if (!gStyleCache->mFormsSheet) {
     nsCOMPtr<nsIURI> sheetURI;
       NS_NewURI(getter_AddRefs(sheetURI),
-                NS_LITERAL_CSTRING("resource://gre/res/forms.css"));
+                NS_LITERAL_CSTRING("resource://gre-resources/forms.css"));
 
     // forms.css needs access to unsafe rules
     if (sheetURI)
-      LoadSheet(sheetURI, gStyleCache->mFormsSheet, PR_TRUE, PR_FALSE);
+      LoadSheet(sheetURI, gStyleCache->mFormsSheet, PR_TRUE);
 
     NS_ASSERTION(gStyleCache->mFormsSheet, "Could not load forms.css.");
   }
@@ -114,7 +115,7 @@ nsLayoutStylesheetCache::FormsSheet()
   return gStyleCache->mFormsSheet;
 }
 
-nsICSSStyleSheet*
+nsCSSStyleSheet*
 nsLayoutStylesheetCache::UserContentSheet()
 {
   EnsureGlobal();
@@ -124,7 +125,7 @@ nsLayoutStylesheetCache::UserContentSheet()
   return gStyleCache->mUserContentSheet;
 }
 
-nsICSSStyleSheet*
+nsCSSStyleSheet*
 nsLayoutStylesheetCache::UserChromeSheet()
 {
   EnsureGlobal();
@@ -134,7 +135,7 @@ nsLayoutStylesheetCache::UserChromeSheet()
   return gStyleCache->mUserChromeSheet;
 }
 
-nsICSSStyleSheet*
+nsCSSStyleSheet*
 nsLayoutStylesheetCache::UASheet()
 {
   EnsureGlobal();
@@ -144,7 +145,7 @@ nsLayoutStylesheetCache::UASheet()
   return gStyleCache->mUASheet;
 }
 
-nsICSSStyleSheet*
+nsCSSStyleSheet*
 nsLayoutStylesheetCache::QuirkSheet()
 {
   EnsureGlobal();
@@ -158,14 +159,13 @@ void
 nsLayoutStylesheetCache::Shutdown()
 {
   NS_IF_RELEASE(gCSSLoader);
-  NS_IF_RELEASE(gCaseSensitiveCSSLoader);
   NS_IF_RELEASE(gStyleCache);
 }
 
 nsLayoutStylesheetCache::nsLayoutStylesheetCache()
 {
   nsCOMPtr<nsIObserverService> obsSvc =
-    do_GetService("@mozilla.org/observer-service;1");
+    mozilla::services::GetObserverService();
   NS_ASSERTION(obsSvc, "No global observer service?");
 
   if (obsSvc) {
@@ -180,15 +180,15 @@ nsLayoutStylesheetCache::nsLayoutStylesheetCache()
   // And make sure that we load our UA sheets.  No need to do this
   // per-profile, since they're profile-invariant.
   nsCOMPtr<nsIURI> uri;
-  NS_NewURI(getter_AddRefs(uri), "resource://gre/res/ua.css");
+  NS_NewURI(getter_AddRefs(uri), "resource://gre-resources/ua.css");
   if (uri) {
-    LoadSheet(uri, mUASheet, PR_TRUE, PR_FALSE);
+    LoadSheet(uri, mUASheet, PR_TRUE);
   }
   NS_ASSERTION(mUASheet, "Could not load ua.css");
 
-  NS_NewURI(getter_AddRefs(uri), "resource://gre/res/quirk.css");
+  NS_NewURI(getter_AddRefs(uri), "resource://gre-resources/quirk.css");
   if (uri) {
-    LoadSheet(uri, mQuirkSheet, PR_TRUE, PR_FALSE);
+    LoadSheet(uri, mQuirkSheet, PR_TRUE);
   }
   NS_ASSERTION(mQuirkSheet, "Could not load quirk.css");
 }
@@ -196,7 +196,6 @@ nsLayoutStylesheetCache::nsLayoutStylesheetCache()
 nsLayoutStylesheetCache::~nsLayoutStylesheetCache()
 {
   gCSSLoader = nsnull;
-  gCaseSensitiveCSSLoader = nsnull;
   gStyleCache = nsnull;
 }
 
@@ -242,7 +241,7 @@ nsLayoutStylesheetCache::InitFromProfile()
 }
 
 void
-nsLayoutStylesheetCache::LoadSheetFile(nsIFile* aFile, nsCOMPtr<nsICSSStyleSheet> &aSheet)
+nsLayoutStylesheetCache::LoadSheetFile(nsIFile* aFile, nsRefPtr<nsCSSStyleSheet> &aSheet)
 {
   PRBool exists = PR_FALSE;
   aFile->Exists(&exists);
@@ -252,40 +251,32 @@ nsLayoutStylesheetCache::LoadSheetFile(nsIFile* aFile, nsCOMPtr<nsICSSStyleSheet
   nsCOMPtr<nsIURI> uri;
   NS_NewFileURI(getter_AddRefs(uri), aFile);
 
-  LoadSheet(uri, aSheet, PR_FALSE, PR_FALSE);
+  LoadSheet(uri, aSheet, PR_FALSE);
 }
 
 void
-nsLayoutStylesheetCache::LoadSheet(nsIURI* aURI, nsCOMPtr<nsICSSStyleSheet> &aSheet,
-                                   PRBool aEnableUnsafeRules,
-                                   PRBool aUseCaseSensitiveLoader)
+nsLayoutStylesheetCache::LoadSheet(nsIURI* aURI,
+                                   nsRefPtr<nsCSSStyleSheet> &aSheet,
+                                   PRBool aEnableUnsafeRules)
 {
   if (!aURI) {
     NS_ERROR("Null URI. Out of memory?");
     return;
   }
 
-  nsICSSLoader** cssLoader =
-    aUseCaseSensitiveLoader ? &gCaseSensitiveCSSLoader : &gCSSLoader;
-
-  if (!*cssLoader) {
-    NS_NewCSSLoader(cssLoader);
-    if (aUseCaseSensitiveLoader) {
-      (*cssLoader)->SetCaseSensitive(PR_TRUE);
-    }
+  if (!gCSSLoader) { 
+    gCSSLoader = new mozilla::css::Loader();
+    NS_IF_ADDREF(gCSSLoader);
   }
 
-  if (*cssLoader) {
-    (*cssLoader)->LoadSheetSync(aURI, aEnableUnsafeRules, PR_TRUE,
-                                getter_AddRefs(aSheet));
+  if (gCSSLoader) {
+    gCSSLoader->LoadSheetSync(aURI, aEnableUnsafeRules, PR_TRUE,
+                              getter_AddRefs(aSheet));
   }
-}  
+}
 
 nsLayoutStylesheetCache*
 nsLayoutStylesheetCache::gStyleCache = nsnull;
 
-nsICSSLoader*
+mozilla::css::Loader*
 nsLayoutStylesheetCache::gCSSLoader = nsnull;
-
-nsICSSLoader*
-nsLayoutStylesheetCache::gCaseSensitiveCSSLoader = nsnull;
