@@ -59,6 +59,12 @@
 #include "stdlib.h"
 #include "nsWildCard.h"
 #include "nsZipArchive.h"
+#include "nsString.h"
+#include "mozilla/FunctionTimer.h"
+#include "prenv.h"
+#if defined(XP_WIN)
+#include <windows.h>
+#endif
 
 /**
  * Global allocator used with zlib. Destroyed in module shutdown.
@@ -77,8 +83,6 @@ nsRecyclingAllocator *gZlibAllocator = NULL;
     #include <unistd.h>
 #elif defined(XP_WIN) || defined(XP_OS2)
     #include <io.h>
-#elif defined(XP_BEOS)
-    #include <unistd.h>
 #endif
 
 #ifdef __SYMBIAN32__
@@ -98,20 +102,18 @@ nsRecyclingAllocator *gZlibAllocator = NULL;
 #  endif
 #endif  /* XP_UNIX */
 
-#include "zipfile.h"
-#include "zipstruct.h"
-#include "nsZipArchive.h"
 
+using namespace mozilla;
 
 static const PRUint32 kMaxNameLength = PATH_MAX; /* Maximum name length */
 // For synthetic zip entries. Date/time corresponds to 1980-01-01 00:00.
 static const PRUint16 kSyntheticTime = 0;
 static const PRUint16 kSyntheticDate = (1 + (1 << 5) + (0 << 9));
 
-static PRUint16 xtoint(const unsigned char *ii);
-static PRUint32 xtolong(const unsigned char *ll);
+static PRUint16 xtoint(const PRUint8 *ii);
+static PRUint32 xtolong(const PRUint8 *ll);
 static PRUint32 HashName(const char* aName, PRUint16 nameLen);
-#if defined(XP_UNIX) || defined(XP_BEOS)
+#ifdef XP_UNIX
 static nsresult ResolveSymlink(const char *path);
 #endif
 
@@ -129,7 +131,7 @@ zlibAlloc(void *opaque, uInt items, uInt size)
 {
   nsRecyclingAllocator *zallocator = (nsRecyclingAllocator *)opaque;
   if (zallocator) {
-    return zallocator->Malloc(items * size);
+    return gZlibAllocator->Malloc(items * size);
   }
   return malloc(items * size);
 }
@@ -163,8 +165,7 @@ nsresult gZlibInit(z_stream *zs)
 }
 
 nsZipHandle::nsZipHandle()
-  : mFd(nsnull)
-  , mFileData(nsnull)
+  : mFileData(nsnull)
   , mLen(0)
   , mMap(nsnull)
   , mRefCnt(0)
@@ -175,8 +176,13 @@ nsZipHandle::nsZipHandle()
 NS_IMPL_THREADSAFE_ADDREF(nsZipHandle)
 NS_IMPL_THREADSAFE_RELEASE(nsZipHandle)
 
-nsresult nsZipHandle::Init(PRFileDesc *fd, nsZipHandle **ret)
+nsresult nsZipHandle::Init(nsILocalFile *file, nsZipHandle **ret)
 {
+  mozilla::AutoFDClose fd;
+  nsresult rv = file->OpenNSPRFileDesc(PR_RDONLY, 0000, &fd);
+  if (NS_FAILED(rv))
+    return rv;
+
   PRInt64 size = PR_Available64(fd);
   if (size >= PR_INT32_MAX)
     return NS_ERROR_FILE_TOO_BIG;
@@ -192,34 +198,51 @@ nsresult nsZipHandle::Init(PRFileDesc *fd, nsZipHandle **ret)
     return NS_ERROR_FAILURE;
   }
 
-  nsZipHandle *handle = new nsZipHandle();
+  nsRefPtr<nsZipHandle> handle = new nsZipHandle();
   if (!handle) {
-    PR_MemUnmap(buf, size);
+    PR_MemUnmap(buf, (PRUint32) size);
     PR_CloseFileMap(map);
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  handle->mFd = fd;
   handle->mMap = map;
+  handle->mFile = file;
   handle->mLen = (PRUint32) size;
   handle->mFileData = buf;
-  handle->AddRef();
-  *ret = handle;
+  *ret = handle.forget().get();
+  return NS_OK;
+}
+
+nsresult nsZipHandle::Init(nsZipArchive *zip, const char *entry,
+                           nsZipHandle **ret)
+{
+  nsRefPtr<nsZipHandle> handle = new nsZipHandle();
+  if (!handle)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  handle->mBuf = new nsZipItemPtr<PRUint8>(zip, entry);
+  if (!handle->mBuf)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  if (!handle->mBuf->Buffer())
+    return NS_ERROR_UNEXPECTED;
+
+  handle->mMap = nsnull;
+  handle->mLen = handle->mBuf->Length();
+  handle->mFileData = handle->mBuf->Buffer();
+  *ret = handle.forget().get();
   return NS_OK;
 }
 
 nsZipHandle::~nsZipHandle()
 {
-  if (mFileData) {
-    PR_MemUnmap(mFileData, mLen);
+  if (mMap) {
+    PR_MemUnmap((void *)mFileData, mLen);
     PR_CloseFileMap(mMap);
-    mFileData = nsnull;
-    mMap = nsnull;
-  }
-  if (mFd) {
-    PR_Close(mFd);
-    mFd = nsnull;
   }
+  mFileData = nsnull;
+  mMap = nsnull;
+  mBuf = nsnull;
   MOZ_COUNT_DTOR(nsZipHandle);
 }
 
@@ -227,21 +250,54 @@ nsZipHandle::~nsZipHandle()
 //      nsZipArchive  --  public methods
 //***********************************************************
 
-
 //---------------------------------------------
 //  nsZipArchive::OpenArchive
 //---------------------------------------------
-nsresult nsZipArchive::OpenArchive(PRFileDesc * fd)
+nsresult nsZipArchive::OpenArchive(nsZipHandle *aZipHandle)
 {
-  nsresult rv = nsZipHandle::Init(fd, getter_AddRefs(mFd));
-  if (NS_FAILED(rv))
-    return rv;
+  mFd = aZipHandle;
 
   // Initialize our arena
   PL_INIT_ARENA_POOL(&mArena, "ZipArena", ZIP_ARENABLOCKSIZE);
 
   //-- get table of contents for archive
-  return BuildFileList();
+  nsresult rv = BuildFileList();
+  char *env = PR_GetEnv("MOZ_JAR_LOG_DIR");
+  if (env && NS_SUCCEEDED(rv) && aZipHandle->mFile) {
+    nsCOMPtr<nsILocalFile> logFile;
+    nsresult rv2 = NS_NewLocalFile(NS_ConvertUTF8toUTF16(env), PR_FALSE, getter_AddRefs(logFile));
+    
+    if (!NS_SUCCEEDED(rv2))
+      return rv;
+
+    // Create a directory for the log (in case it doesn't exist)
+    logFile->Create(nsIFile::DIRECTORY_TYPE, 0700);
+
+    nsAutoString name;
+    aZipHandle->mFile->GetLeafName(name);
+    name.Append(NS_LITERAL_STRING(".log"));
+    logFile->Append(name);
+
+    PRFileDesc* fd;
+    rv2 = logFile->OpenNSPRFileDesc(PR_WRONLY|PR_CREATE_FILE|PR_APPEND, 0644, &fd);
+    if (NS_SUCCEEDED(rv2))
+      mLog = fd;
+  }
+  return rv;
+}
+
+nsresult nsZipArchive::OpenArchive(nsIFile *aFile)
+{
+  nsresult rv;
+  nsCOMPtr<nsILocalFile> localFile = do_QueryInterface(aFile, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsRefPtr<nsZipHandle> handle;
+  rv = nsZipHandle::Init(localFile, getter_AddRefs(handle));
+  if (NS_FAILED(rv))
+    return rv;
+
+  return OpenArchive(handle);
 }
 
 //---------------------------------------------
@@ -314,16 +370,26 @@ nsZipItem*  nsZipArchive::GetItem(const char * aEntryName)
                 return 0;
         }
     }
-
+MOZ_WIN_MEM_TRY_BEGIN
     nsZipItem* item = mFiles[ HashName(aEntryName, len) ];
     while (item) {
       if ((len == item->nameLength) && 
-         (!memcmp(aEntryName, item->Name(), len)))
+          (!memcmp(aEntryName, item->Name(), len))) {
+        
+        if (mLog) {
+          // Successful GetItem() is a good indicator that the file is about to be read
+          char *tmp = PL_strdup(aEntryName);
+          tmp[len]='\n';
+          PR_Write(mLog, tmp, len+1);
+          PL_strfree(tmp);
+        }
         return item; //-- found it
+      }
       item = item->next;
     }
+MOZ_WIN_MEM_TRY_CATCH(return nsnull)
   }
-  return 0;
+  return nsnull;
 }
 
 //---------------------------------------------
@@ -345,22 +411,26 @@ nsresult nsZipArchive::ExtractFile(nsZipItem *item, const char *outname,
   // so the item to be extracted should never be a directory
   PR_ASSERT(!item->IsDirectory());
 
-  nsresult rv;
+  Bytef outbuf[ZIP_BUFLEN];
 
-  //-- extract the file using the appropriate method
-  switch(item->Compression())
-  {
-    case STORED:
-      rv = CopyItemToDisk(item, aFd);
+  nsZipCursor cursor(item, this, outbuf, ZIP_BUFLEN, true);
+
+  nsresult rv = NS_OK;
+
+  while (true) {
+    PRUint32 count = 0;
+    PRUint8* buf = cursor.Read(&count);
+    if (!buf) {
+      rv = NS_ERROR_FILE_CORRUPTED;
       break;
-
-    case DEFLATED:
-      rv = InflateItem(item, aFd);
+    } else if (count == 0) {
       break;
+    }
 
-    default:
-      //-- unsupported compression type
-      rv = NS_ERROR_NOT_IMPLEMENTED;
+    if (aFd && PR_Write(aFd, buf, count) < (READTYPE)count) {
+      rv = NS_ERROR_FILE_DISK_FULL;
+      break;
+    }
   }
 
   //-- delete the file on errors, or resolve symlink if needed
@@ -368,7 +438,7 @@ nsresult nsZipArchive::ExtractFile(nsZipItem *item, const char *outname,
     PR_Close(aFd);
     if (rv != NS_OK)
       PR_Delete(outname);
-#if defined(XP_UNIX) || defined(XP_BEOS)
+#ifdef XP_UNIX
     else if (item->IsSymlink())
       rv = ResolveSymlink(outname);
 #endif
@@ -445,7 +515,7 @@ nsresult nsZipFind::FindNext(const char ** aResult, PRUint16 *aNameLen)
 
   *aResult = 0;
   *aNameLen = 0;
-
+MOZ_WIN_MEM_TRY_BEGIN
   // we start from last match, look for next
   while (mSlot < ZIP_TABSIZE)
   {
@@ -474,11 +544,11 @@ nsresult nsZipFind::FindNext(const char ** aResult, PRUint16 *aNameLen)
       return NS_OK;
     }
   }
-
+MOZ_WIN_MEM_TRY_CATCH(return NS_ERROR_FAILURE)
   return NS_ERROR_FILE_TARGET_DOES_NOT_EXIST;
 }
 
-#if defined(XP_UNIX) || defined(XP_BEOS)
+#ifdef XP_UNIX
 //---------------------------------------------
 // ResolveSymlink
 //---------------------------------------------
@@ -522,18 +592,25 @@ nsZipItem* nsZipArchive::CreateZipItem()
 //---------------------------------------------
 nsresult nsZipArchive::BuildFileList()
 {
+#ifndef XP_WIN
+  NS_TIME_FUNCTION;
+#endif
   // Get archive size using end pos
-  PRUint8* buf;
-  PRUint8* startp = mFd->mFileData;
-  PRUint8* endp = startp + mFd->mLen;
-
-  PRUint32 centralOffset = 0;
-  for (buf = endp - ZIPEND_SIZE; buf > startp; buf--)
-  {
-    if (xtolong(buf) == ENDSIG) {
-      centralOffset = xtolong(((ZipEnd *)buf)->offset_central_dir);
-      break;
-    }
+  const PRUint8* buf;
+  const PRUint8* startp = mFd->mFileData;
+  const PRUint8* endp = startp + mFd->mLen;
+MOZ_WIN_MEM_TRY_BEGIN
+  PRUint32 centralOffset = 4;
+  if (mFd->mLen > ZIPCENTRAL_SIZE && xtolong(startp + centralOffset) == CENTRALSIG) {
+    // Success means optimized jar layout from bug 559961 is in effect
+  } else {
+    for (buf = endp - ZIPEND_SIZE; buf > startp; buf--)
+      {
+        if (xtolong(buf) == ENDSIG) {
+          centralOffset = xtolong(((ZipEnd *)buf)->offset_central_dir);
+          break;
+        }
+      }
   }
 
   if (!centralOffset)
@@ -582,6 +659,8 @@ nsresult nsZipArchive::BuildFileList()
 
   if (sig != ENDSIG)
     return NS_ERROR_FILE_CORRUPTED;
+
+MOZ_WIN_MEM_TRY_CATCH(return NS_ERROR_FAILURE)
   return NS_OK;
 }
 
@@ -594,6 +673,7 @@ nsresult nsZipArchive::BuildSynthetics()
     return NS_OK;
   mBuiltSynthetics = true;
 
+MOZ_WIN_MEM_TRY_BEGIN
   // Create synthetic entries for any missing directories.
   // Do this when all ziptable has scanned to prevent double entries.
   for (int i = 0; i < ZIP_TABSIZE; ++i)
@@ -650,6 +730,7 @@ nsresult nsZipArchive::BuildSynthetics()
       } /* end processing of dirs in item's name */
     }
   }
+MOZ_WIN_MEM_TRY_CATCH(return NS_ERROR_FAILURE)
   return NS_OK;
 }
 
@@ -663,14 +744,14 @@ nsZipHandle* nsZipArchive::GetFD()
 //---------------------------------------------
 // nsZipArchive::GetData
 //---------------------------------------------
-PRUint8* nsZipArchive::GetData(nsZipItem* aItem)
+const PRUint8* nsZipArchive::GetData(nsZipItem* aItem)
 {
   PR_ASSERT (aItem);
-
+MOZ_WIN_MEM_TRY_BEGIN
   //-- read local header to get variable length values and calculate
   //-- the real data offset
   PRUint32 len = mFd->mLen;
-  PRUint8* data = mFd->mFileData;
+  const PRUint8* data = mFd->mFileData;
   PRUint32 offset = aItem->LocalOffset();
   if (offset + ZIPLOCAL_SIZE > len)
     return nsnull;
@@ -692,97 +773,7 @@ PRUint8* nsZipArchive::GetData(nsZipItem* aItem)
     return nsnull;
 
   return data + offset;
-}
-
-//---------------------------------------------
-// nsZipArchive::CopyItemToDisk
-//---------------------------------------------
-nsresult
-nsZipArchive::CopyItemToDisk(nsZipItem *item, PRFileDesc* outFD)
-{
-  PR_ASSERT(item);
-
-  //-- get to the start of file's data
-  const PRUint8* itemData = GetData(item);
-  if (!itemData)
-    return NS_ERROR_FILE_CORRUPTED;
-
-  if (outFD && PR_Write(outFD, itemData, item->Size()) < (READTYPE)item->Size())
-  {
-    //-- Couldn't write all the data (disk full?)
-    return NS_ERROR_FILE_DISK_FULL;
-  }
-
-  //-- Calculate crc
-  PRUint32 crc = crc32(0L, (const unsigned char*)itemData, item->Size());
-  //-- verify crc32
-  if (crc != item->CRC32())
-      return NS_ERROR_FILE_CORRUPTED;
-
-  return NS_OK;
-}
-
-
-//---------------------------------------------
-// nsZipArchive::InflateItem
-//---------------------------------------------
-nsresult nsZipArchive::InflateItem(nsZipItem * item, PRFileDesc* outFD)
-/*
- * This function inflates an archive item to disk, to the
- * file specified by outFD. If outFD is zero, the extracted data is
- * not written, only checked for CRC, so this is in effect same as 'Test'.
- */
-{
-  PR_ASSERT(item);
-  //-- allocate deflation buffers
-  Bytef outbuf[ZIP_BUFLEN];
-
-  //-- set up the inflate
-  z_stream    zs;
-  nsresult status = gZlibInit(&zs);
-  if (status != NS_OK)
-    return NS_ERROR_FAILURE;
-
-  //-- inflate loop
-  zs.avail_in = item->Size();
-  zs.next_in = (Bytef*)GetData(item);
-  if (!zs.next_in)
-    return NS_ERROR_FILE_CORRUPTED;
-
-  PRUint32  crc = crc32(0L, Z_NULL, 0);
-  int zerr = Z_OK;
-  while (zerr == Z_OK)
-  {
-    zs.next_out = outbuf;
-    zs.avail_out = ZIP_BUFLEN;
-
-    zerr = inflate(&zs, Z_PARTIAL_FLUSH);
-    if (zerr != Z_OK && zerr != Z_STREAM_END)
-    {
-      status = (zerr == Z_MEM_ERROR) ? NS_ERROR_OUT_OF_MEMORY : NS_ERROR_FILE_CORRUPTED;
-      break;
-    }
-    PRUint32 count = zs.next_out - outbuf;
-
-    //-- incrementally update crc32
-    crc = crc32(crc, (const unsigned char*)outbuf, count);
-
-    if (outFD && PR_Write(outFD, outbuf, count) < (READTYPE)count)
-    {
-      status = NS_ERROR_FILE_DISK_FULL;
-      break;
-    }
-  } // while
-
-  //-- free zlib internal state
-  inflateEnd(&zs);
-
-  //-- verify crc32
-  if ((status == NS_OK) && (crc != item->CRC32()))
-  {
-    status = NS_ERROR_FILE_CORRUPTED;
-  }
-  return status;
+MOZ_WIN_MEM_TRY_CATCH(return nsnull)
 }
 
 //------------------------------------------
@@ -856,7 +847,7 @@ static PRUint32 HashName(const char* aName, PRUint16 len)
  *  Converts a two byte ugly endianed integer
  *  to our platform's integer.
  */
-static PRUint16 xtoint (const unsigned char *ii)
+static PRUint16 xtoint (const PRUint8 *ii)
 {
   return (PRUint16) ((ii [0]) | (ii [1] << 8));
 }
@@ -867,7 +858,7 @@ static PRUint16 xtoint (const unsigned char *ii)
  *  Converts a four byte ugly endianed integer
  *  to our platform's integer.
  */
-static PRUint32 xtolong (const unsigned char *ll)
+static PRUint32 xtolong (const PRUint8 *ll)
 {
   return (PRUint32)( (ll [0] <<  0) |
                      (ll [1] <<  8) |
@@ -875,56 +866,225 @@ static PRUint32 xtolong (const unsigned char *ll)
                      (ll [3] << 24) );
 }
 
-PRUint32 const nsZipItem::LocalOffset()
+/*
+ * GetModTime
+ *
+ * returns last modification time in microseconds
+ */
+static PRTime GetModTime(PRUint16 aDate, PRUint16 aTime)
+{
+  // Note that on DST shift we can't handle correctly the hour that is valid
+  // in both DST zones
+  PRExplodedTime time;
+
+  time.tm_usec = 0;
+
+  time.tm_hour = (aTime >> 11) & 0x1F;
+  time.tm_min = (aTime >> 5) & 0x3F;
+  time.tm_sec = (aTime & 0x1F) * 2;
+
+  time.tm_year = (aDate >> 9) + 1980;
+  time.tm_month = ((aDate >> 5) & 0x0F) - 1;
+  time.tm_mday = aDate & 0x1F;
+
+  time.tm_params.tp_gmt_offset = 0;
+  time.tm_params.tp_dst_offset = 0;
+
+  PR_NormalizeTime(&time, PR_GMTParameters);
+  time.tm_params.tp_gmt_offset = PR_LocalTimeParameters(&time).tp_gmt_offset;
+  PR_NormalizeTime(&time, PR_GMTParameters);
+  time.tm_params.tp_dst_offset = PR_LocalTimeParameters(&time).tp_dst_offset;
+
+  return PR_ImplodeTime(&time);
+}
+
+PRUint32 nsZipItem::LocalOffset()
 {
   return xtolong(central->localhdr_offset);
 }
 
-PRUint32 const nsZipItem::Size()
+PRUint32 nsZipItem::Size()
 {
   return isSynthetic ? 0 : xtolong(central->size);
 }
 
-PRUint32 const nsZipItem::RealSize()
+PRUint32 nsZipItem::RealSize()
 {
   return isSynthetic ? 0 : xtolong(central->orglen);
 }
 
-PRUint32 const nsZipItem::CRC32()
+PRUint32 nsZipItem::CRC32()
 {
   return isSynthetic ? 0 : xtolong(central->crc32);
 }
 
-PRUint16 const nsZipItem::Date()
+PRUint16 nsZipItem::Date()
 {
   return isSynthetic ? kSyntheticDate : xtoint(central->date);
 }
 
-PRUint16 const nsZipItem::Time()
+PRUint16 nsZipItem::Time()
 {
   return isSynthetic ? kSyntheticTime : xtoint(central->time);
 }
 
-PRUint16 const nsZipItem::Compression()
+PRUint16 nsZipItem::Compression()
 {
   return isSynthetic ? STORED : xtoint(central->method);
 }
 
-bool const nsZipItem::IsDirectory()
+bool nsZipItem::IsDirectory()
 {
   return isSynthetic || ((nameLength > 0) && ('/' == Name()[nameLength - 1]));
 }
 
-PRUint16 const nsZipItem::Mode()
+PRUint16 nsZipItem::Mode()
 {
   if (isSynthetic) return 0755;
   return ((PRUint16)(central->external_attributes[2]) | 0x100);
 }
 
-#if defined(XP_UNIX) || defined(XP_BEOS)
-bool const nsZipItem::IsSymlink()
+const PRUint8 * nsZipItem::GetExtraField(PRUint16 aTag, PRUint16 *aBlockSize)
+{
+  if (isSynthetic) return nsnull;
+MOZ_WIN_MEM_TRY_BEGIN
+  const unsigned char *buf = ((const unsigned char*)central) + ZIPCENTRAL_SIZE +
+                             nameLength;
+  PRUint32 buflen = (PRUint32)xtoint(central->extrafield_len);
+  PRUint32 pos = 0;
+  PRUint16 tag, blocksize;
+
+  while (buf && (pos + 4) <= buflen) {
+    tag = xtoint(buf + pos);
+    blocksize = xtoint(buf + pos + 2);
+
+    if (aTag == tag && (pos + 4 + blocksize) <= buflen) {
+      *aBlockSize = blocksize;
+      return buf + pos;
+    }
+
+    pos += blocksize + 4;
+  }
+
+MOZ_WIN_MEM_TRY_CATCH(return nsnull)
+  return nsnull;
+}
+
+
+PRTime nsZipItem::LastModTime()
+{
+  if (isSynthetic) return GetModTime(kSyntheticDate, kSyntheticTime);
+
+  // Try to read timestamp from extra field
+  PRUint16 blocksize;
+  const PRUint8 *tsField = GetExtraField(EXTENDED_TIMESTAMP_FIELD, &blocksize);
+  if (tsField && blocksize >= 5 && tsField[4] & EXTENDED_TIMESTAMP_MODTIME) {
+    return (PRTime)(xtolong(tsField + 5)) * PR_USEC_PER_SEC;
+  }
+
+  return GetModTime(Date(), Time());
+}
+
+#ifdef XP_UNIX
+bool nsZipItem::IsSymlink()
 {
   if (isSynthetic) return false;
   return (xtoint(central->external_attributes+2) & S_IFMT) == S_IFLNK;
 }
 #endif
+
+nsZipCursor::nsZipCursor(nsZipItem *item, nsZipArchive *aZip, PRUint8* aBuf, PRUint32 aBufSize, bool doCRC) :
+  mItem(item),
+  mBuf(aBuf),
+  mBufSize(aBufSize),
+  mDoCRC(doCRC)
+{
+  if (mItem->Compression() == DEFLATED) {
+#ifdef DEBUG
+    nsresult status =
+#endif
+      gZlibInit(&mZs);
+    NS_ASSERTION(status == NS_OK, "Zlib failed to initialize");
+    NS_ASSERTION(aBuf, "Must pass in a buffer for DEFLATED nsZipItem");
+  }
+  
+  mZs.avail_in = item->Size();
+  mZs.next_in = (Bytef*)aZip->GetData(item);
+  
+  if (doCRC)
+    mCRC = crc32(0L, Z_NULL, 0);
+}
+
+nsZipCursor::~nsZipCursor()
+{
+  if (mItem->Compression() == DEFLATED) {
+    inflateEnd(&mZs);
+  }
+}
+
+PRUint8* nsZipCursor::Read(PRUint32 *aBytesRead) {
+  int zerr;
+  PRUint8 *buf = nsnull;
+  bool verifyCRC = true;
+
+  if (!mZs.next_in)
+    return nsnull;
+MOZ_WIN_MEM_TRY_BEGIN
+  switch (mItem->Compression()) {
+  case STORED:
+    *aBytesRead = mZs.avail_in;
+    buf = mZs.next_in;
+    mZs.next_in += mZs.avail_in;
+    mZs.avail_in = 0;
+    break;
+  case DEFLATED:
+    buf = mBuf;
+    mZs.next_out = buf;
+    mZs.avail_out = mBufSize;
+    
+    zerr = inflate(&mZs, Z_PARTIAL_FLUSH);
+    if (zerr != Z_OK && zerr != Z_STREAM_END)
+      return nsnull;
+    
+    *aBytesRead = mZs.next_out - buf;
+    verifyCRC = (zerr == Z_STREAM_END);
+    break;
+  default:
+    return nsnull;
+  }
+
+  if (mDoCRC) {
+    mCRC = crc32(mCRC, (const unsigned char*)buf, *aBytesRead);
+    if (verifyCRC && mCRC != mItem->CRC32())
+      return nsnull;
+  }
+MOZ_WIN_MEM_TRY_CATCH(return nsnull)
+  return buf;
+}
+
+nsZipItemPtr_base::nsZipItemPtr_base(nsZipArchive *aZip, const char * aEntryName, bool doCRC) :
+  mReturnBuf(nsnull)
+{
+  // make sure the ziparchive hangs around
+  mZipHandle = aZip->GetFD();
+
+  nsZipItem* item = aZip->GetItem(aEntryName);
+  if (!item)
+    return;
+
+  PRUint32 size = 0;
+  if (item->Compression() == DEFLATED) {
+    size = item->RealSize();
+    mAutoBuf = new PRUint8[size];
+  }
+
+  nsZipCursor cursor(item, aZip, mAutoBuf, size, doCRC);
+  mReturnBuf = cursor.Read(&mReadlen);
+  if (!mReturnBuf)
+    return;
+
+  if (mReadlen != item->RealSize()) {
+    NS_ASSERTION(mReadlen == item->RealSize(), "nsZipCursor underflow");
+    mReturnBuf = nsnull;
+  }
+}
