@@ -76,9 +76,9 @@
 
 #include "mozilla/FunctionTimer.h"
 
-#ifdef MOZ_IPC
 #include "mozilla/net/NeckoCommon.h"
-#endif
+
+using namespace mozilla;
 
 /******************************************************************************
  * nsCacheProfilePrefObserver
@@ -275,12 +275,11 @@ public:
     }
     NS_IMETHOD Run()
     {
-        mozilla::MonitorAutoEnter
-            autoMonitor(nsCacheService::gService->mMonitor);
+        nsCacheServiceAutoLock autoLock;
 #ifdef PR_LOGGING
         CACHE_LOG_DEBUG(("nsBlockOnCacheThreadEvent [%p]\n", this));
 #endif
-        autoMonitor.Notify();
+        nsCacheService::gService->mCondVar.Notify();
         return NS_OK;
     }
 };
@@ -809,12 +808,9 @@ nsCacheService::DispatchToCacheIOThread(nsIRunnable* event)
 nsresult
 nsCacheService::SyncWithCacheIOThread()
 {
-    NS_ASSERTION(gService->mLockedThread == PR_GetCurrentThread(),
-                 "not holding cache-lock");
+    gService->mLock.AssertCurrentThreadOwns();
     if (!gService->mCacheIOThread) return NS_ERROR_NOT_AVAILABLE;
 
-    mozilla::MonitorAutoEnter autoMonitor(gService->mMonitor);
-
     nsCOMPtr<nsIRunnable> event = new nsBlockOnCacheThreadEvent();
 
     // dispatch event - it will notify the monitor when it's done
@@ -825,10 +821,8 @@ nsCacheService::SyncWithCacheIOThread()
         return NS_ERROR_UNEXPECTED;
     }
 
-    Unlock();
     // wait until notified, then return
-    rv = autoMonitor.Wait();
-    Lock();
+    rv = gService->mCondVar.Wait();
 
     return rv;
 }
@@ -985,8 +979,8 @@ nsCacheService *   nsCacheService::gService = nsnull;
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsCacheService, nsICacheService)
 
 nsCacheService::nsCacheService()
-    : mLock(nsnull),
-      mMonitor("block-on-cache-monitor"),
+    : mLock("nsCacheService.mLock"),
+      mCondVar(mLock, "nsCacheService.mCondVar"),
       mInitialized(PR_FALSE),
       mEnableMemoryDevice(PR_TRUE),
       mEnableDiskDevice(PR_TRUE),
@@ -1007,13 +1001,6 @@ nsCacheService::nsCacheService()
 
     // create list of cache devices
     PR_INIT_CLIST(&mDoomedEntries);
-  
-    // allocate service lock
-    mLock = PR_NewLock();
-
-#if defined(DEBUG)
-    mLockedThread = nsnull;
-#endif
 }
 
 nsCacheService::~nsCacheService()
@@ -1021,7 +1008,6 @@ nsCacheService::~nsCacheService()
     if (mInitialized) // Shutdown hasn't been called yet.
         (void) Shutdown();
 
-    PR_DestroyLock(mLock);
     gService = nsnull;
 }
 
@@ -1035,14 +1021,9 @@ nsCacheService::Init()
     if (mInitialized)
         return NS_ERROR_ALREADY_INITIALIZED;
 
-#ifdef MOZ_IPC
     if (mozilla::net::IsNeckoChild()) {
         return NS_ERROR_UNEXPECTED;
     }
-#endif
-
-    if (mLock == nsnull)
-        return NS_ERROR_OUT_OF_MEMORY;
 
     CACHE_LOG_INIT();
 
@@ -1637,7 +1618,6 @@ nsCacheService::OpenCacheEntry(nsCacheSession *           session,
 
     nsCacheRequest * request = nsnull;
 
-    nsCacheServiceAutoLock lock;
     nsresult rv = gService->CreateRequest(session,
                                           key,
                                           accessRequested,
@@ -1653,17 +1633,15 @@ nsCacheService::OpenCacheEntry(nsCacheSession *           session,
     if (NS_IsMainThread() && listener && gService->mCacheIOThread) {
         nsCOMPtr<nsIRunnable> ev =
             new nsProcessRequestEvent(request);
-        if (ev) {
-            rv = gService->mCacheIOThread->Dispatch(ev, NS_DISPATCH_NORMAL);
-        } else {
-            rv = NS_ERROR_OUT_OF_MEMORY;
-        }
+        rv = DispatchToCacheIOThread(ev);
 
         // delete request if we didn't post the event
         if (NS_FAILED(rv))
             delete request;
     }
     else {
+
+        nsCacheServiceAutoLock lock;
         rv = gService->ProcessRequest(request, PR_TRUE, result);
 
         // delete requests that have completed
@@ -1849,6 +1827,7 @@ nsCacheService::EnsureEntryHasDevice(nsCacheEntry * entry)
         if (mDiskDevice) {
             // Bypass the cache if Content-Length says the entry will be too big
             if (predictedDataSize != -1 &&
+                entry->StoragePolicy() != nsICache::STORE_ON_DISK_AS_FILE &&
                 mDiskDevice->EntryIsTooBig(predictedDataSize)) {
                 nsresult rv = nsCacheService::DoomEntry(entry);
                 NS_ASSERTION(NS_SUCCEEDED(rv),"DoomEntry() failed.");
@@ -2214,25 +2193,18 @@ nsCacheService::OnDataSizeChange(nsCacheEntry * entry, PRInt32 deltaSize)
 void
 nsCacheService::Lock()
 {
-    PR_Lock(gService->mLock);
-
-#if defined(DEBUG)
-    gService->mLockedThread = PR_GetCurrentThread();
-#endif
+    gService->mLock.Lock();
 }
 
 void
 nsCacheService::Unlock()
 {
-    NS_ASSERTION(gService->mLockedThread == PR_GetCurrentThread(), "oops");
+    gService->mLock.AssertCurrentThreadOwns();
 
     nsTArray<nsISupports*> doomed;
     doomed.SwapElements(gService->mDoomedObjects);
 
-#if defined(DEBUG)
-    gService->mLockedThread = nsnull;
-#endif
-    PR_Unlock(gService->mLock);
+    gService->mLock.Unlock();
 
     for (PRUint32 i = 0; i < doomed.Length(); ++i)
         doomed[i]->Release();
@@ -2242,7 +2214,7 @@ void
 nsCacheService::ReleaseObject_Locked(nsISupports * obj,
                                      nsIEventTarget * target)
 {
-    NS_ASSERTION(gService->mLockedThread == PR_GetCurrentThread(), "oops");
+    gService->mLock.AssertCurrentThreadOwns();
 
     PRBool isCur;
     if (!target || (NS_SUCCEEDED(target->IsOnCurrentThread(&isCur)) && isCur)) {
