@@ -47,10 +47,6 @@
  */
 #define NS_VIEWSOURCE_TOKENS_PER_BLOCK 16
 
-// TODO get rid of these unused macros
-#define STOP_TIMER()
-#define START_TIMER()
-
 #include "nsIAtom.h"
 #include "nsViewSourceHTML.h"
 #include "nsCRT.h"
@@ -98,6 +94,7 @@ Stopwatch vsTimer;
 // viewsource.css; the setting is remembered between invocations using a pref.
 static const char kBodyId[] = "viewsource";
 static const char kBodyClassWrap[] = "wrap";
+static const char kBodyTabSize[] = "-moz-tab-size: ";
 
 NS_IMPL_ISUPPORTS1(CViewSourceHTML, nsIDTD)
 
@@ -201,6 +198,7 @@ CViewSourceHTML::CViewSourceHTML()
 {
   mSyntaxHighlight = PR_FALSE;
   mWrapLongLines = PR_FALSE;
+  mTabSize = -1;
   nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
   if (prefBranch) {
     PRBool temp;
@@ -210,6 +208,9 @@ CViewSourceHTML::CViewSourceHTML()
 
     rv = prefBranch->GetBoolPref("view_source.wrap_long_lines", &temp);
     mWrapLongLines = NS_SUCCEEDED(rv) ? temp : PR_FALSE;
+
+    rv = prefBranch->GetIntPref("view_source.tab_size", &temp);
+    mTabSize = NS_SUCCEEDED(rv) ? temp : -1;
   }
 
   mSink = 0;
@@ -261,7 +262,6 @@ CViewSourceHTML::WillBuildModel(const CParserContext& aParserContext,
   NS_START_STOPWATCH(vsTimer);
 #endif
 
-  STOP_TIMER();
   mSink=(nsIHTMLContentSink*)aSink;
 
   if((!aParserContext.mPrevContext) && (mSink)) {
@@ -286,7 +286,7 @@ CViewSourceHTML::WillBuildModel(const CParserContext& aParserContext,
       fprintf(gDumpFile, "Source of: ");
       fputs(NS_ConvertUTF16toUTF8(mFilename).get(), gDumpFile);
       fprintf(gDumpFile, "</title>\n");
-      fprintf(gDumpFile, "<link rel=\"stylesheet\" type=\"text/css\" href=\"resource://gre/res/viewsource.css\">\n");
+      fprintf(gDumpFile, "<link rel=\"stylesheet\" type=\"text/css\" href=\"resource://gre-resources/viewsource.css\">\n");
       fprintf(gDumpFile, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n");
       fprintf(gDumpFile, "</head>\n");
       fprintf(gDumpFile, "<body id=\"viewsource\">\n");
@@ -302,8 +302,6 @@ CViewSourceHTML::WillBuildModel(const CParserContext& aParserContext,
 
   mLineNumber = 1;
 
-  START_TIMER();
-
   return result;
 }
 
@@ -378,7 +376,7 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsITokenizer* aTokenizer,
 
           AddAttrToNode(theNode, theAllocator,
                         NS_LITERAL_STRING("href"),
-                        NS_LITERAL_STRING("resource://gre/res/viewsource.css"));
+                        NS_LITERAL_STRING("resource://gre-resources/viewsource.css"));
 
           mSink->AddLeaf(theNode);
         }
@@ -409,6 +407,13 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsITokenizer* aTokenizer,
                           NS_LITERAL_STRING("class"),
                           NS_ConvertASCIItoUTF16(kBodyClassWrap));
           }
+          if (mTabSize >= 0) {
+            nsAutoString styleValue = NS_ConvertASCIItoUTF16(kBodyTabSize);
+            styleValue.AppendInt(mTabSize);
+            AddAttrToNode(bodyNode, theAllocator,
+                          NS_LITERAL_STRING("style"),
+                          styleValue);
+          }
           result = mSink->OpenContainer(bodyNode);
           if(NS_SUCCEEDED(result)) mHasOpenBody=PR_TRUE;
         }
@@ -540,8 +545,6 @@ NS_IMETHODIMP CViewSourceHTML::DidBuildModel(nsresult anErrorCode)
 
   //ADD CODE HERE TO CLOSE OPEN CONTAINERS...
 
-  STOP_TIMER();
-
   if (mSink) {
       //now let's close automatically auto-opened containers...
 
@@ -561,8 +564,6 @@ NS_IMETHODIMP CViewSourceHTML::DidBuildModel(nsresult anErrorCode)
     }
   }
 
-  START_TIMER();
-
 #ifdef RAPTOR_PERF_METRICS
   NS_STOP_STOPWATCH(vsTimer);
   printf("viewsource timer: ");
@@ -769,8 +770,6 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsSubstring & aText,PR
 #endif // DUMP_TO_FILE
   }
 
-  STOP_TIMER();
-
   mITextToken.SetIndirectString(aText);  //now emit the tag name...
 
   nsCParserNode theNode(&mITextToken, 0/*stack token*/);
@@ -816,8 +815,6 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsSubstring & aText,PR
 #endif //DUMP_TO_FILE
   }
 
-  START_TIMER();
-
   return result;
 }
 
@@ -963,6 +960,8 @@ PRBool CViewSourceHTML::IsUrlAttribute(const nsAString& tagName,
 
   PRBool isHref = trimmedAttrName.LowerCaseEqualsLiteral("href");
   PRBool isSrc = !isHref && trimmedAttrName.LowerCaseEqualsLiteral("src");
+  PRBool isXLink = !isHref && !isSrc &&
+    mDocType == eXML && trimmedAttrName.EqualsLiteral("xlink:href");
 
   // If this is the HREF attribute of a BASE element, then update the base URI.
   // This doesn't feel like the ideal place for this, but the alternatives don't
@@ -974,7 +973,7 @@ PRBool CViewSourceHTML::IsUrlAttribute(const nsAString& tagName,
     SetBaseURI(expandedBaseSpec);
   }
 
-  return isHref || isSrc;
+  return isHref || isSrc || isXLink;
 }
 
 void CViewSourceHTML::WriteHrefAttribute(nsTokenAllocator* allocator,
