@@ -39,9 +39,7 @@
 #include "nsQAppInstance.h"
 #endif
 
-#ifdef MOZ_IPC
 #include "base/basictypes.h"
-#endif
 
 #include "nsXULAppAPI.h"
 
@@ -79,7 +77,6 @@
 #include "nsXREDirProvider.h"
 
 #include "mozilla/Omnijar.h"
-#ifdef MOZ_IPC
 #if defined(XP_MACOSX)
 #include "chrome/common/mach_ipc_mac.h"
 #endif
@@ -135,7 +132,6 @@ using mozilla::ipc::TestShellCommandParent;
 using mozilla::ipc::XPCShellEnvironment;
 
 using mozilla::startup::sChildProcessType;
-#endif
 
 static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
 
@@ -249,7 +245,6 @@ XRE_StringToChildProcessType(const char* aProcessTypeString)
   return GeckoProcessType_Invalid;
 }
 
-#ifdef MOZ_IPC
 namespace mozilla {
 namespace startup {
 GeckoProcessType sChildProcessType = GeckoProcessType_Default;
@@ -773,6 +768,3 @@ XRE_InstallX11ErrorHandler()
   InstallX11ErrorHandler();
 }
 #endif
-
-#endif // MOZ_IPC
-
