@@ -80,6 +80,19 @@
 
 #include "nsDocLoader.h"
 
+#include "mozilla/FunctionTimer.h"
+#ifdef NS_FUNCTION_TIMER
+#define TIME_URILOADER_FUNCTION(req)                         \
+    nsCAutoString name__("N/A");                             \
+    (req)->GetName(name__);                                  \
+    NS_TIME_FUNCTION_FMT("%s (line %d) (request: %s)",       \
+                         MOZ_FUNCTION_NAME,                  \
+                         __LINE__,                           \
+                         name__.get())
+#else
+#define TIME_URILOADER_FUNCTION(req) do {} while(0)
+#endif
+
 #ifdef PR_LOGGING
 PRLogModuleInfo* nsURILoader::mLog = nsnull;
 #endif
@@ -224,6 +237,8 @@ nsresult nsDocumentOpenInfo::Prepare()
 
 NS_IMETHODIMP nsDocumentOpenInfo::OnStartRequest(nsIRequest *request, nsISupports * aCtxt)
 {
+  TIME_URILOADER_FUNCTION(request);
+
   LOG(("[0x%p] nsDocumentOpenInfo::OnStartRequest", this));
   
   nsresult rv = NS_OK;
@@ -297,6 +312,8 @@ NS_IMETHODIMP nsDocumentOpenInfo::OnStartRequest(nsIRequest *request, nsISupport
 NS_IMETHODIMP nsDocumentOpenInfo::OnDataAvailable(nsIRequest *request, nsISupports * aCtxt,
                                                   nsIInputStream * inStr, PRUint32 sourceOffset, PRUint32 count)
 {
+  TIME_URILOADER_FUNCTION(request);
+
   // if we have retarged to the end stream listener, then forward the call....
   // otherwise, don't do anything
 
@@ -310,6 +327,8 @@ NS_IMETHODIMP nsDocumentOpenInfo::OnDataAvailable(nsIRequest *request, nsISuppor
 NS_IMETHODIMP nsDocumentOpenInfo::OnStopRequest(nsIRequest *request, nsISupports *aCtxt, 
                                                 nsresult aStatus)
 {
+  TIME_URILOADER_FUNCTION(request);
+
   LOG(("[0x%p] nsDocumentOpenInfo::OnStopRequest", this));
   
   if ( m_targetStreamListener)
@@ -332,6 +351,8 @@ NS_IMETHODIMP nsDocumentOpenInfo::OnStopRequest(nsIRequest *request, nsISupports
 
 nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest *request, nsISupports * aCtxt)
 {
+  TIME_URILOADER_FUNCTION(request);
+
   LOG(("[0x%p] nsDocumentOpenInfo::DispatchContent for type '%s'", this, mContentType.get()));
 
   NS_PRECONDITION(!m_targetStreamListener,
@@ -437,7 +458,8 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest *request, nsISupports *
       return NS_OK;
     }
 
-    // If we aren't allowed to try other listeners, we're done here.
+    // If we aren't allowed to try other listeners, just skip through to
+    // trying to convert the data.
     if (!(mFlags & nsIURILoader::DONT_RETARGET)) {
 
       //
@@ -516,44 +538,39 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest *request, nsISupports *
           return rv;
         }
       }
+    } else {
+      LOG(("  DONT_RETARGET flag set, so skipped over random other content "
+           "listeners and content handlers"));
+    }
+
+    //
+    // Fifth step:  If no listener prefers this type, see if any stream
+    //              converters exist to transform this content type into
+    //              some other.
+    //
+    // Don't do this if the server sent us a MIME type of "*/*" because they saw
+    // it in our Accept header and got confused.
+    // XXXbz have to be careful here; may end up in some sort of bizarre infinite
+    // decoding loop.
+    if (mContentType != anyType) {
+      rv = ConvertData(request, m_contentListener, mContentType, anyType);
+      if (NS_FAILED(rv)) {
+        m_targetStreamListener = nsnull;
+      } else if (m_targetStreamListener) {
+        // We found a converter for this MIME type.  We'll just pump data into it
+        // and let the downstream nsDocumentOpenInfo handle things.
+        LOG(("  Converter taking over now"));
+        return NS_OK;
+      }
     }
-  } else if (mFlags & nsIURILoader::DONT_RETARGET) {
-    // External handling was forced, but we must not retarget
-    // -> abort
-    LOG(("  External handling forced, but not allowed to retarget -> aborting"));
-    return NS_ERROR_WONT_HANDLE_CONTENT;
   }
 
   NS_ASSERTION(!m_targetStreamListener,
                "If we found a listener, why are we not using it?");
   
-  //
-  // Fifth step:  If no listener prefers this type, see if any stream
-  //              converters exist to transform this content type into
-  //              some other.
-  //
-
-  // We always want to do this, since even content being forced to
-  // be handled externally may need decoding (eg via the unknown
-  // content decoder).
-  // Don't do this if the server sent us a MIME type of "*/*" because they saw
-  // it in our Accept header and got confused.
-  // XXXbz have to be careful here; may end up in some sort of bizarre infinite
-  // decoding loop.
-  if (mContentType != anyType) {
-    rv = ConvertData(request, m_contentListener, mContentType, anyType);
-    if (NS_FAILED(rv)) {
-      m_targetStreamListener = nsnull;
-    } else if (m_targetStreamListener) {
-      // We found a converter for this MIME type.  We'll just pump data into it
-      // and let the downstream nsDocumentOpenInfo handle things.
-      LOG(("  Converter taking over now"));
-      return NS_OK;
-    }
-  }
-
   if (mFlags & nsIURILoader::DONT_RETARGET) {
-    LOG(("  Listener not interested and no stream converter exists, and retargeting disallowed -> aborting"));
+    LOG(("  External handling forced or (listener not interested and no "
+         "stream converter exists), and retargeting disallowed -> aborting"));
     return NS_ERROR_WONT_HANDLE_CONTENT;
   }
 
@@ -613,6 +630,8 @@ nsDocumentOpenInfo::ConvertData(nsIRequest *request,
                                 const nsACString& aSrcContentType,
                                 const nsACString& aOutContentType)
 {
+  TIME_URILOADER_FUNCTION(request);
+
   LOG(("[0x%p] nsDocumentOpenInfo::ConvertData from '%s' to '%s'", this,
        PromiseFlatCString(aSrcContentType).get(),
        PromiseFlatCString(aOutContentType).get()));
@@ -669,6 +688,8 @@ PRBool
 nsDocumentOpenInfo::TryContentListener(nsIURIContentListener* aListener,
                                        nsIChannel* aChannel)
 {
+  TIME_URILOADER_FUNCTION(aChannel);
+
   LOG(("[0x%p] nsDocumentOpenInfo::TryContentListener; mFlags = 0x%x",
        this, mFlags));
 
@@ -813,6 +834,8 @@ NS_IMETHODIMP nsURILoader::OpenURI(nsIChannel *channel,
 {
   NS_ENSURE_ARG_POINTER(channel);
 
+  TIME_URILOADER_FUNCTION(channel);
+
 #ifdef PR_LOGGING
   if (LOG_ENABLED()) {
     nsCOMPtr<nsIURI> uri;
@@ -860,6 +883,8 @@ nsresult nsURILoader::OpenChannel(nsIChannel* channel,
   NS_ASSERTION(channel, "Trying to open a null channel!");
   NS_ASSERTION(aWindowContext, "Window context must not be null");
 
+  TIME_URILOADER_FUNCTION(channel);
+
 #ifdef PR_LOGGING
   if (LOG_ENABLED()) {
     nsCOMPtr<nsIURI> uri;
